@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2009 the original author or authors.
+ * Copyright 2008-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -36,20 +36,19 @@ import org.objectweb.asm.Opcodes;
 import java.beans.PropertyVetoException;
 import java.beans.VetoableChangeListener;
 import java.beans.VetoableChangeSupport;
-import java.util.Collection;
 
 /**
  * Handles generation of code for the {@code @Vetoable} annotation, and {@code @Bindable}
  * if also present.
- * <p/>
+ * <p>
  * Generally, it adds (if needed) a VetoableChangeSupport field and
  * the needed add/removeVetoableChangeListener methods to support the
  * listeners.
- * <p/>
+ * <p>
  * It also generates the setter and wires the setter through the
  * VetoableChangeSupport.
- * <p/>
- * If a {@link Bindable} annotaton is detected it also adds support similar
+ * <p>
+ * If a {@link Bindable} annotation is detected it also adds support similar
  * to what {@link BindableASTTransformation} would do.
  *
  * @author Danno Ferrin (shemnon)
@@ -58,8 +57,8 @@ import java.util.Collection;
 @GroovyASTTransformation(phase = CompilePhase.CANONICALIZATION)
 public class VetoableASTTransformation extends BindableASTTransformation {
 
-    protected static ClassNode constrainedClassNode = new ClassNode(Vetoable.class);
-    protected ClassNode vcsClassNode = new ClassNode(VetoableChangeSupport.class);
+    protected static ClassNode constrainedClassNode = ClassHelper.make(Vetoable.class);
+    protected ClassNode vcsClassNode = ClassHelper.make(VetoableChangeSupport.class);
 
     /**
      * Convenience method to see if an annotated node is {@code @Vetoable}.
@@ -68,7 +67,7 @@ public class VetoableASTTransformation extends BindableASTTransformation {
      * @return true if the node is constrained
      */
     public static boolean hasVetoableAnnotation(AnnotatedNode node) {
-        for (AnnotationNode annotation : (Collection<AnnotationNode>) node.getAnnotations()) {
+        for (AnnotationNode annotation : node.getAnnotations()) {
             if (constrainedClassNode.equals(annotation.getClassNode())) {
                 return true;
             }
@@ -92,12 +91,10 @@ public class VetoableASTTransformation extends BindableASTTransformation {
             addListenerToClass(source, node, (ClassNode) nodes[1]);
         } else {
             if ((((FieldNode)nodes[1]).getModifiers() & Opcodes.ACC_FINAL) != 0) {
-                source.getErrorCollector().addErrorAndContinue(
-                            new SyntaxErrorMessage(new SyntaxException(
-                                "@groovy.beans.Vetoable cannot annotate a final property.",
-                                node.getLineNumber(),
-                                node.getColumnNumber()),
-                                source));
+                source.getErrorCollector().addErrorAndContinue(new SyntaxErrorMessage(
+                        new SyntaxException("@groovy.beans.Vetoable cannot annotate a final property.",
+                                node.getLineNumber(), node.getColumnNumber(), node.getLastLineNumber(), node.getLastColumnNumber()),
+                        source));
             }
 
             addListenerToProperty(source, node, (AnnotatedNode) nodes[1]);
@@ -108,38 +105,34 @@ public class VetoableASTTransformation extends BindableASTTransformation {
         ClassNode declaringClass = parent.getDeclaringClass();
         FieldNode field = ((FieldNode) parent);
         String fieldName = field.getName();
-        for (PropertyNode propertyNode : (Collection<PropertyNode>) declaringClass.getProperties()) {
+        for (PropertyNode propertyNode : declaringClass.getProperties()) {
             boolean bindable = BindableASTTransformation.hasBindableAnnotation(parent)
-                || BindableASTTransformation.hasBindableAnnotation(parent.getDeclaringClass());
+                    || BindableASTTransformation.hasBindableAnnotation(parent.getDeclaringClass());
 
             if (propertyNode.getName().equals(fieldName)) {
                 if (field.isStatic()) {
                     //noinspection ThrowableInstanceNeverThrown
-                    source.getErrorCollector().addErrorAndContinue(
-                                new SyntaxErrorMessage(new SyntaxException(
-                                    "@groovy.beans.Vetoable cannot annotate a static property.",
-                                    node.getLineNumber(),
-                                    node.getColumnNumber()),
-                                    source));
+                    source.getErrorCollector().addErrorAndContinue(new SyntaxErrorMessage(
+                            new SyntaxException("@groovy.beans.Vetoable cannot annotate a static property.",
+                                    node.getLineNumber(), node.getColumnNumber(), node.getLastLineNumber(), node.getLastColumnNumber()),
+                            source));
                 } else {
-                    createListenerSetter(source, node, bindable, declaringClass,  propertyNode);
+                    createListenerSetter(source, node, bindable, declaringClass, propertyNode);
                 }
                 return;
             }
         }
         //noinspection ThrowableInstanceNeverThrown
-        source.getErrorCollector().addErrorAndContinue(
-                    new SyntaxErrorMessage(new SyntaxException(
-                        "@groovy.beans.Vetoable must be on a property, not a field.  Try removing the private, protected, or public modifier.",
-                        node.getLineNumber(),
-                        node.getColumnNumber()),
-                        source));
+        source.getErrorCollector().addErrorAndContinue(new SyntaxErrorMessage(
+                new SyntaxException("@groovy.beans.Vetoable must be on a property, not a field.  Try removing the private, protected, or public modifier.",
+                        node.getLineNumber(), node.getColumnNumber(), node.getLastLineNumber(), node.getLastColumnNumber()),
+                source));
     }
 
 
     private void addListenerToClass(SourceUnit source, AnnotationNode node, ClassNode classNode) {
         boolean bindable = BindableASTTransformation.hasBindableAnnotation(classNode);
-        for (PropertyNode propertyNode : (Collection<PropertyNode>) classNode.getProperties()) {
+        for (PropertyNode propertyNode : classNode.getProperties()) {
             if (!hasVetoableAnnotation(propertyNode.getField())
                 && !((propertyNode.getField().getModifiers() & Opcodes.ACC_FINAL) != 0)
                 && !propertyNode.getField().isStatic())
@@ -235,7 +228,7 @@ public class VetoableASTTransformation extends BindableASTTransformation {
     }
 
     /**
-     * Creates a statement body silimar to:
+     * Creates a statement body similar to:
      * <code>this.fireVetoableChange("field", field, field = value)</code>
      *
      * @param propertyNode           the field node for the property
@@ -256,9 +249,9 @@ public class VetoableASTTransformation extends BindableASTTransformation {
 
     /**
      * Creates a statement body similar to:
-     * <code>field = value</code>
-     * <p/>
-     * Used when the field is not also @Bindable
+     * <code>field = value</code>.
+     * <p>
+     * Used when the field is not also {@code @Bindable}.
      *
      * @param fieldExpression a field expression for setting the property value
      * @return the created statement
@@ -295,6 +288,15 @@ public class VetoableASTTransformation extends BindableASTTransformation {
             }
             consideredClass = consideredClass.getSuperClass();
         }
+        // check if a super class has @Vetoable annotations
+        consideredClass = declaringClass.getSuperClass();
+        while (consideredClass!=null) {
+            if (hasVetoableAnnotation(consideredClass)) return false;
+            for (FieldNode field : consideredClass.getFields()) {
+                if (hasVetoableAnnotation(field)) return false;
+            }
+            consideredClass = consideredClass.getSuperClass();
+        }
         if (foundAdd || foundRemove || foundFire) {
             sourceUnit.getErrorCollector().addErrorAndContinue(
                 new SimpleMessage("@Vetoable cannot be processed on "
@@ -309,7 +311,7 @@ public class VetoableASTTransformation extends BindableASTTransformation {
 
     /**
      * Creates a setter method with the given body.
-     * <p/>
+     * <p>
      * This differs from normal setters in that we need to add a declared
      * exception java.beans.PropertyVetoException
      *
@@ -320,7 +322,7 @@ public class VetoableASTTransformation extends BindableASTTransformation {
      */
     protected void createSetterMethod(ClassNode declaringClass, PropertyNode propertyNode, String setterName, Statement setterBlock) {
         Parameter[] setterParameterTypes = {new Parameter(propertyNode.getType(), "value")};
-        ClassNode[] exceptions = {new ClassNode(PropertyVetoException.class)};
+        ClassNode[] exceptions = {ClassHelper.make(PropertyVetoException.class)};
         MethodNode setter =
                 new MethodNode(setterName, propertyNode.getModifiers(), ClassHelper.VOID_TYPE, setterParameterTypes, exceptions, setterBlock);
         setter.setSynthetic(true);
@@ -330,10 +332,10 @@ public class VetoableASTTransformation extends BindableASTTransformation {
 
     /**
      * Adds the necessary field and methods to support vetoable change support.
-     * <p/>
+     * <p>
      * Adds a new field:
      * <code>"protected final java.beans.VetoableChangeSupport this$vetoableChangeSupport = new java.beans.VetoableChangeSupport(this)"</code>
-     * <p/>
+     * <p>
      * Also adds support methods:
      * <code>public void addVetoableChangeListener(java.beans.VetoableChangeListener)</code>
      * <code>public void addVetoableChangeListener(String, java.beans.VetoableChangeListener)</code>
@@ -437,7 +439,7 @@ public class VetoableASTTransformation extends BindableASTTransformation {
                         ACC_PUBLIC | ACC_SYNTHETIC,
                         ClassHelper.VOID_TYPE,
                         new Parameter[]{new Parameter(ClassHelper.STRING_TYPE, "name"), new Parameter(ClassHelper.OBJECT_TYPE, "oldValue"), new Parameter(ClassHelper.OBJECT_TYPE, "newValue")},
-                        new ClassNode[] {new ClassNode(PropertyVetoException.class)},
+                        new ClassNode[] {ClassHelper.make(PropertyVetoException.class)},
                         new ExpressionStatement(
                                 new MethodCallExpression(
                                         new FieldExpression(vcsField),
@@ -486,4 +488,4 @@ public class VetoableASTTransformation extends BindableASTTransformation {
                                                 new Expression[]{new VariableExpression("name")}))))));
     }
 
-}
\ No newline at end of file
+}
