@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2011 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,30 +17,47 @@
 package groovy.grape;
 
 import groovy.lang.Grab;
+import groovy.lang.GrabConfig;
+import groovy.lang.GrabExclude;
 import groovy.lang.GrabResolver;
 import groovy.lang.Grapes;
-import groovy.lang.GrabExclude;
-import groovy.lang.GrabConfig;
-import org.codehaus.groovy.ast.*;
-import org.codehaus.groovy.ast.expr.*;
+import groovy.transform.CompilationUnitAware;
+import org.codehaus.groovy.ast.ASTNode;
+import org.codehaus.groovy.ast.AnnotatedNode;
+import org.codehaus.groovy.ast.AnnotationNode;
+import org.codehaus.groovy.ast.ClassCodeVisitorSupport;
+import org.codehaus.groovy.ast.ClassHelper;
+import org.codehaus.groovy.ast.ClassNode;
+import org.codehaus.groovy.ast.ImportNode;
+import org.codehaus.groovy.ast.ModuleNode;
+import org.codehaus.groovy.ast.expr.ArgumentListExpression;
+import org.codehaus.groovy.ast.expr.ConstantExpression;
+import org.codehaus.groovy.ast.expr.Expression;
+import org.codehaus.groovy.ast.expr.ListExpression;
+import org.codehaus.groovy.ast.expr.MapExpression;
+import org.codehaus.groovy.ast.expr.MethodCallExpression;
+import org.codehaus.groovy.ast.expr.StaticMethodCallExpression;
+import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
+import org.codehaus.groovy.control.CompilationUnit;
 import org.codehaus.groovy.control.CompilePhase;
 import org.codehaus.groovy.control.SourceUnit;
+import org.codehaus.groovy.runtime.DefaultGroovyMethods;
+import org.codehaus.groovy.tools.GrapeUtil;
 import org.codehaus.groovy.transform.ASTTransformation;
 import org.codehaus.groovy.transform.ASTTransformationVisitor;
 import org.codehaus.groovy.transform.GroovyASTTransformation;
-import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 
 import java.util.*;
-import java.util.regex.Pattern;
 import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 /**
  * Transformation for declarative dependency management.
  */
 @GroovyASTTransformation(phase=CompilePhase.CONVERSION)
-public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implements ASTTransformation {
+public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implements ASTTransformation, CompilationUnitAware {
     private static final String GRAB_CLASS_NAME = Grab.class.getName();
     private static final String GRAB_DOT_NAME = GRAB_CLASS_NAME.substring(GRAB_CLASS_NAME.lastIndexOf("."));
     private static final String GRAB_SHORT_NAME = GRAB_DOT_NAME.substring(1);
@@ -58,12 +75,13 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
     private static final String GRAPES_SHORT_NAME = shortName(GRAPES_DOT_NAME);
 
     private static final String GRABRESOLVER_CLASS_NAME = GrabResolver.class.getName();
-    private static final String GRAPERESOLVER_DOT_NAME = dotName(GRABRESOLVER_CLASS_NAME);
-    private static final String GRABRESOLVER_SHORT_NAME = shortName(GRAPERESOLVER_DOT_NAME);
+    private static final String GRABRESOLVER_DOT_NAME = dotName(GRABRESOLVER_CLASS_NAME);
+    private static final String GRABRESOLVER_SHORT_NAME = shortName(GRABRESOLVER_DOT_NAME);
 
     private static final ClassNode THREAD_CLASSNODE = ClassHelper.make(Thread.class);
+
     private static final List<String> GRABEXCLUDE_REQUIRED = Arrays.asList("group", "module");
-    private static final List<String> GRAPERESOLVER_REQUIRED = Arrays.asList("name", "root");
+    private static final List<String> GRABRESOLVER_REQUIRED = Arrays.asList("name", "root");
     private static final List<String> GRAB_REQUIRED = Arrays.asList("group", "module", "version");
     private static final List<String> GRAB_OPTIONAL = Arrays.asList("classifier", "transitive", "conf", "ext", "type", "changing", "force", "initClass");
     private static final List<String> GRAB_BOOLEAN = Arrays.asList("transitive", "changing", "force", "initClass");
@@ -71,6 +89,9 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
     private static final Pattern IVY_PATTERN = Pattern.compile("([a-zA-Z0-9-/._+=]+)#([a-zA-Z0-9-/._+=]+)(;([a-zA-Z0-9-/.\\(\\)\\[\\]\\{\\}_+=,:@][a-zA-Z0-9-/.\\(\\)\\]\\{\\}_+=,:@]*))?(\\[([a-zA-Z0-9-/._+=,]*)\\])?");
     private static final Pattern ATTRIBUTES_PATTERN = Pattern.compile("(.*;|^)([a-zA-Z0-9]+)=([a-zA-Z0-9.*\\[\\]\\-\\(\\),]*)$");
 
+    private static final String AUTO_DOWNLOAD_SETTING = Grape.AUTO_DOWNLOAD_SETTING;
+    private static final String DISABLE_CHECKSUMS_SETTING = Grape.DISABLE_CHECKSUMS_SETTING;
+
     private static String dotName(String className) {
         return className.substring(className.lastIndexOf("."));
     }
@@ -99,14 +120,21 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
     Set<String> grabResolverAliases;
     List<AnnotationNode> grabResolverAnnotations;
 
+    CompilationUnit compilationUnit;
     SourceUnit sourceUnit;
     ClassLoader loader;
     boolean initContextClassLoader;
+    Boolean autoDownload;
+    Boolean disableChecksums;
 
     public SourceUnit getSourceUnit() {
         return sourceUnit;
     }
 
+    public void setCompilationUnit(final CompilationUnit compilationUnit) {
+        this.compilationUnit = compilationUnit;
+    }
+
     public void visit(ASTNode[] nodes, SourceUnit source) {
         sourceUnit = source;
         loader = null;
@@ -141,7 +169,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
             } else if (GRAPES_CLASS_NAME.equals(className)) {
                 grapesAliases.add(im.getAlias());
             }
-            if ((className.endsWith(GRAPERESOLVER_DOT_NAME) && ((alias == null) || (alias.length() == 0)))
+            if ((className.endsWith(GRABRESOLVER_DOT_NAME) && ((alias == null) || (alias.length() == 0)))
                 || (GRABRESOLVER_CLASS_NAME.equals(alias)))
             {
                 allowShortGrabResolver = false;
@@ -151,6 +179,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
         }
 
         List<Map<String,Object>> grabMaps = new ArrayList<Map<String,Object>>();
+        List<Map<String,Object>> grabMapsInit = new ArrayList<Map<String,Object>>();
         List<Map<String,Object>> grabExcludeMaps = new ArrayList<Map<String,Object>>();
 
         for (ClassNode classNode : sourceUnit.getAST().getClasses()) {
@@ -164,45 +193,7 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
 
             ClassNode grapeClassNode = ClassHelper.make(Grape.class);
 
-            if (!grabResolverAnnotations.isEmpty()) {
-                grabResolverAnnotationLoop:
-                for (AnnotationNode node : grabResolverAnnotations) {
-                    Map<String, Object> grapeResolverMap = new HashMap<String, Object>();
-                    Expression value = node.getMember("value");
-                    ConstantExpression ce = null;
-                    if (value != null && value instanceof ConstantExpression) {
-                        ce = (ConstantExpression) value;
-                    }
-                    String sval = null;
-                    if (ce != null && ce.getValue() instanceof String) {
-                        sval = (String) ce.getValue();
-                    }
-                    if (sval != null && sval.length() > 0) {
-                        for (String s : GRAPERESOLVER_REQUIRED) {
-                            Expression member = node.getMember(s);
-                            if (member != null) {
-                                addError("The attribute \"" + s + "\" conflicts with attribute 'value' in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
-                                continue grabResolverAnnotationLoop;
-                            }
-                        }
-                        grapeResolverMap.put("name", sval);
-                        grapeResolverMap.put("root", sval);
-                    } else {
-                        for (String s : GRAPERESOLVER_REQUIRED) {
-                            Expression member = node.getMember(s);
-                            if (member == null) {
-                                addError("The missing attribute \"" + s + "\" is required in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
-                                continue grabResolverAnnotationLoop;
-                            } else if (member != null && !(member instanceof ConstantExpression)) {
-                                addError("Attribute \"" + s + "\" has value " + member.getText() + " but should be an inline constant in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
-                                continue grabResolverAnnotationLoop;
-                            }
-                            grapeResolverMap.put(s, ((ConstantExpression) member).getValue());
-                        }
-                    }
-                    Grape.addResolver(grapeResolverMap);
-                }
-            }
+            List<Statement> grabResolverInitializers = new ArrayList<Statement>();
 
             if (!grapesAnnotations.isEmpty()) {
                 for (AnnotationNode node : grapesAnnotations) {
@@ -223,10 +214,53 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
                 }
             }
 
+            if (!grabResolverAnnotations.isEmpty()) {
+                grabResolverAnnotationLoop:
+                for (AnnotationNode node : grabResolverAnnotations) {
+                    Map<String, Object> grabResolverMap = new HashMap<String, Object>();
+                    Expression value = node.getMember("value");
+                    ConstantExpression ce = null;
+                    if (value != null && value instanceof ConstantExpression) {
+                        ce = (ConstantExpression) value;
+                    }
+                    String sval = null;
+                    if (ce != null && ce.getValue() instanceof String) {
+                        sval = (String) ce.getValue();
+                    }
+                    if (sval != null && sval.length() > 0) {
+                        for (String s : GRABRESOLVER_REQUIRED) {
+                            Expression member = node.getMember(s);
+                            if (member != null) {
+                                addError("The attribute \"" + s + "\" conflicts with attribute 'value' in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
+                                continue grabResolverAnnotationLoop;
+                            }
+                        }
+                        grabResolverMap.put("name", sval);
+                        grabResolverMap.put("root", sval);
+                    } else {
+                        for (String s : GRABRESOLVER_REQUIRED) {
+                            Expression member = node.getMember(s);
+                            if (member == null) {
+                                addError("The missing attribute \"" + s + "\" is required in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
+                                continue grabResolverAnnotationLoop;
+                            } else if (member != null && !(member instanceof ConstantExpression)) {
+                                addError("Attribute \"" + s + "\" has value " + member.getText() + " but should be an inline constant in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
+                                continue grabResolverAnnotationLoop;
+                            }
+                            grabResolverMap.put(s, ((ConstantExpression) member).getValue());
+                        }
+                    }
+                    Grape.addResolver(grabResolverMap);
+                    addGrabResolverAsStaticInitIfNeeded(grapeClassNode, node, grabResolverInitializers, grabResolverMap);
+                }
+            }
+
             if (!grabConfigAnnotations.isEmpty()) {
                 for (AnnotationNode node : grabConfigAnnotations) {
                     checkForClassLoader(node);
                     checkForInitContextClassLoader(node);
+                    checkForAutoDownload(node);
+                    checkForDisableChecksums(node);
                 }
                 addInitContextClassLoaderIfNeeded(classNode);
             }
@@ -265,12 +299,21 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
                             addError("Attribute \"" + s + "\" has value " + member.getText() + " but should be an inline constant in @" + node.getClassNode().getNameWithoutPackage() + " annotations", node);
                             continue grabAnnotationLoop;
                         }
-                        if (node.getMember(s) != null)
+                        if (node.getMember(s) != null) {
                             grabMap.put(s, ((ConstantExpression)member).getValue());
+                        }
                     }
                     grabMaps.add(grabMap);
-                    callGrabAsStaticInitIfNeeded(classNode, grapeClassNode, node, grabExcludeMaps);
+                    if ((node.getMember("initClass") == null)
+                            || (node.getMember("initClass") == ConstantExpression.TRUE)) {
+                        grabMapsInit.add(grabMap);
+                    }
                 }
+                callGrabAsStaticInitIfNeeded(classNode, grapeClassNode, grabMapsInit, grabExcludeMaps);
+            }
+
+            if (!grabResolverInitializers.isEmpty()) {
+                classNode.addStaticInitializerStatements(grabResolverInitializers, true);
             }
         }
 
@@ -278,11 +321,15 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
             Map<String, Object> basicArgs = new HashMap<String, Object>();
             basicArgs.put("classLoader", loader != null ? loader : sourceUnit.getClassLoader());
             if (!grabExcludeMaps.isEmpty()) basicArgs.put("excludes", grabExcludeMaps);
+            if (autoDownload != null) basicArgs.put(AUTO_DOWNLOAD_SETTING, autoDownload);
+            if (disableChecksums != null) basicArgs.put(DISABLE_CHECKSUMS_SETTING, disableChecksums);
 
             try {
                 Grape.grab(basicArgs, grabMaps.toArray(new Map[grabMaps.size()]));
                 // grab may have added more transformations through new URLs added to classpath, so do one more scan
-                ASTTransformationVisitor.addGlobalTransformsAfterGrab();
+                if (compilationUnit!=null) {
+                    ASTTransformationVisitor.addGlobalTransformsAfterGrab(compilationUnit.getASTTransformationsContext());
+                }
             } catch (RuntimeException re) {
                 // Decided against syntax exception since this is not a syntax error.
                 // The down side is we lose line number information for the offending
@@ -292,46 +339,62 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
         }
     }
 
-    private void callGrabAsStaticInitIfNeeded(ClassNode classNode, ClassNode grapeClassNode, AnnotationNode node, List<Map<String, Object>> grabExcludeMaps) {
+    private void callGrabAsStaticInitIfNeeded(ClassNode classNode, ClassNode grapeClassNode, List<Map<String,Object>> grabMapsInit, List<Map<String, Object>> grabExcludeMaps) {
+        List<Statement> grabInitializers = new ArrayList<Statement>();
+        MapExpression basicArgs = new MapExpression();
+        if (autoDownload != null)  {
+            basicArgs.addMapEntryExpression(new ConstantExpression(AUTO_DOWNLOAD_SETTING), new ConstantExpression(autoDownload));
+        }
+
+        if (disableChecksums != null)  {
+            basicArgs.addMapEntryExpression(new ConstantExpression(DISABLE_CHECKSUMS_SETTING), new ConstantExpression(disableChecksums));
+        }
+        if (!grabExcludeMaps.isEmpty()) {
+            ListExpression list = new ListExpression();
+            for (Map<String, Object> map : grabExcludeMaps) {
+                Set<Map.Entry<String, Object>> entries = map.entrySet();
+                MapExpression inner = new MapExpression();
+                for (Map.Entry<String, Object> entry : entries) {
+                    inner.addMapEntryExpression(new ConstantExpression(entry.getKey()), new ConstantExpression(entry.getValue()));
+                }
+                list.addExpression(inner);
+            }
+            basicArgs.addMapEntryExpression(new ConstantExpression("excludes"), list);
+        }
+
+        List<Expression> argList = new ArrayList<Expression>();
+        argList.add(basicArgs);
+        for (Map<String, Object> grabMap : grabMapsInit) {
+            // add Grape.grab(excludeArgs, [group:group, module:module, version:version, classifier:classifier])
+            // or Grape.grab([group:group, module:module, version:version, classifier:classifier])
+            MapExpression dependencyArg = new MapExpression();
+            for (String s : GRAB_REQUIRED) {
+                dependencyArg.addMapEntryExpression(new ConstantExpression(s), new ConstantExpression(grabMap.get(s)));
+            }
+            for (String s : GRAB_OPTIONAL) {
+                if (grabMap.containsKey(s))
+                    dependencyArg.addMapEntryExpression(new ConstantExpression(s), new ConstantExpression(grabMap.get(s)));
+            }
+            argList.add(dependencyArg);
+        }
+        ArgumentListExpression grabArgs = new ArgumentListExpression(argList);
+        grabInitializers.add(new ExpressionStatement(new StaticMethodCallExpression(grapeClassNode, "grab", grabArgs)));
+
+        // insert at beginning so we have the classloader set up before the class is called
+        classNode.addStaticInitializerStatements(grabInitializers, true);
+    }
+
+    private void addGrabResolverAsStaticInitIfNeeded(ClassNode grapeClassNode, AnnotationNode node,
+                                                      List<Statement> grabResolverInitializers, Map<String, Object> grabResolverMap) {
         if ((node.getMember("initClass") == null)
             || (node.getMember("initClass") == ConstantExpression.TRUE))
         {
-            List<Statement> grabInitializers = new ArrayList<Statement>();
-
-            // add Grape.grab(excludeArgs, [group:group, module:module, version:version, classifier:classifier])
-            // or Grape.grab([group:group, module:module, version:version, classifier:classifier])
-            MapExpression me = new MapExpression();
-            for (String s : GRAB_REQUIRED) {
-                me.addMapEntryExpression(new ConstantExpression(s),node.getMember(s));
+            MapExpression resolverArgs = new MapExpression();
+            for (Map.Entry<String, Object> next : grabResolverMap.entrySet()) {
+                resolverArgs.addMapEntryExpression(new ConstantExpression(next.getKey()), new ConstantExpression(next.getValue()));
             }
-
-            for (String s : GRAB_OPTIONAL) {
-                if (node.getMember(s) != null)
-                    me.addMapEntryExpression(new ConstantExpression(s),node.getMember(s));
-            }
-
-            ArgumentListExpression grabArgs;
-            if (grabExcludeMaps.isEmpty()) {
-                grabArgs = new ArgumentListExpression(me);
-            } else {
-                MapExpression args = new MapExpression();
-                ListExpression list = new ListExpression();
-                for (Map<String, Object> map : grabExcludeMaps) {
-                    Set<Map.Entry<String, Object>> entries = map.entrySet();
-                    MapExpression inner = new MapExpression();
-                    for (Map.Entry<String, Object> entry : entries) {
-                        inner.addMapEntryExpression(new ConstantExpression(entry.getKey()), new ConstantExpression(entry.getValue()));
-                    }
-                    list.addExpression(inner);
-                }
-                args.addMapEntryExpression(new ConstantExpression("excludes"), list);
-                grabArgs = new ArgumentListExpression(args, me);
-            }
-            grabInitializers.add(new ExpressionStatement(
-                    new StaticMethodCallExpression(grapeClassNode, "grab", grabArgs)));
-
-            // insert at beginning so we have the classloader set up before the class is called
-            classNode.addStaticInitializerStatements(grabInitializers, true);
+            grabResolverInitializers.add(new ExpressionStatement(
+                    new StaticMethodCallExpression(grapeClassNode, "addResolver", new ArgumentListExpression(resolverArgs))));
         }
     }
 
@@ -370,6 +433,22 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
         initContextClassLoader = (Boolean) initContextClassLoaderObject;
     }
 
+    private void checkForAutoDownload(AnnotationNode node) {
+        Object val = node.getMember(AUTO_DOWNLOAD_SETTING);
+        if (val == null || !(val instanceof ConstantExpression)) return;
+        Object autoDownloadValue = ((ConstantExpression)val).getValue();
+        if (!(autoDownloadValue instanceof Boolean)) return;
+        autoDownload = (Boolean) autoDownloadValue;
+    }
+
+    private void checkForDisableChecksums(AnnotationNode node) {
+        Object val = node.getMember(DISABLE_CHECKSUMS_SETTING);
+        if (val == null || !(val instanceof ConstantExpression)) return;
+        Object disableChecksumsValue = ((ConstantExpression)val).getValue();
+        if (!(disableChecksumsValue instanceof Boolean)) return;
+        disableChecksums = (Boolean) disableChecksumsValue;
+    }
+
     private void checkForConvenienceForm(AnnotationNode node, boolean exclude) {
         Object val = node.getMember("value");
         if (val == null || !(val instanceof ConstantExpression)) return;
@@ -414,22 +493,13 @@ public class GrabAnnotationTransformation extends ClassCodeVisitorSupport implem
         } else if (allstr.contains(":")) {
             // assume gradle syntax
             // see: http://www.gradle.org/latest/docs/userguide/dependency_management.html#sec:how_to_declare_your_dependencies
-            String ext = "";
-            String[] parts;
-            if (allstr.contains("@")) {
-                parts = allstr.split("@");
-                if (parts.length > 2) return;
-                allstr = parts[0];
-                ext = parts[1];
+            Map<String, Object> parts = GrapeUtil.getIvyParts(allstr);
+            for (String key : parts.keySet()) {
+                String value = parts.get(key).toString();
+                if (!key.equals("version") || !value.equals("*") || !exclude) {
+                    node.addMember(key, new ConstantExpression(value));
+                }
             }
-            parts = allstr.split(":");
-            if (parts.length > 4) return;
-            if (parts.length > 3) node.addMember("classifier", new ConstantExpression(parts[3]));
-            if (parts.length > 2) node.addMember("version", new ConstantExpression(parts[2]));
-            else if (!exclude && node.getMember("version") == null) node.addMember("version", new ConstantExpression("*"));
-            if (ext.length() > 0) node.addMember("ext", new ConstantExpression(ext));
-            node.addMember("module", new ConstantExpression(parts[1]));
-            node.addMember("group", new ConstantExpression(parts[0]));
             node.getMembers().remove("value");
         }
     }
