@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2012 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,13 +15,14 @@
  */
 package groovy.lang;
 
-import org.codehaus.groovy.runtime.DefaultGroovyMethods;
 import org.codehaus.groovy.runtime.InvokerHelper;
+import org.codehaus.groovy.runtime.StringGroovyMethods;
 
 import java.io.IOException;
 import java.io.Serializable;
 import java.io.StringWriter;
 import java.io.Writer;
+import java.io.UnsupportedEncodingException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
@@ -37,7 +38,7 @@ import java.util.regex.Pattern;
  * couldn't resist :)
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 13100 $
+ * @version $Revision$
  */
 public abstract class GString extends GroovyObjectSupport implements Comparable, CharSequence, Writable, Buildable, Serializable {
 
@@ -63,6 +64,7 @@ public abstract class GString extends GroovyObjectSupport implements Comparable,
     }
 
     // will be static in an instance
+
     public abstract String[] getStrings();
 
     /**
@@ -85,17 +87,17 @@ public abstract class GString extends GroovyObjectSupport implements Comparable,
     }
 
     public GString plus(GString that) {
-        List stringList = new ArrayList();
-        List valueList = new ArrayList();
+        List<String> stringList = new ArrayList<String>();
+        List<Object> valueList = new ArrayList<Object>();
 
         stringList.addAll(Arrays.asList(getStrings()));
         valueList.addAll(Arrays.asList(getValues()));
 
-        List thatStrings = Arrays.asList(that.getStrings());
+        List<String> thatStrings = Arrays.asList(that.getStrings());
         if (stringList.size() > valueList.size()) {
-            thatStrings = new ArrayList(thatStrings);
+            thatStrings = new ArrayList<String>(thatStrings);
             // merge onto end of previous GString to avoid an empty bridging value
-            String s = (String) stringList.get(stringList.size() - 1);
+            String s = stringList.get(stringList.size() - 1);
             s += thatStrings.get(0);
             thatStrings.remove(0);
             stringList.set(stringList.size() - 1, s);
@@ -117,8 +119,8 @@ public abstract class GString extends GroovyObjectSupport implements Comparable,
 
     public GString plus(String that) {
         String[] currentStrings = getStrings();
-        String[] newStrings = null;
-        Object[] newValues = null;
+        String[] newStrings;
+        Object[] newValues;
 
         boolean appendToLastString = currentStrings.length > getValues().length;
 
@@ -166,38 +168,39 @@ public abstract class GString extends GroovyObjectSupport implements Comparable,
     }
 
     public Writer writeTo(Writer out) throws IOException {
-    	String[] s = getStrings();
-    	int numberOfValues = values.length;
-    	for (int i = 0, size = s.length; i < size; i++) {
-    		out.write(s[i]);
-    		if (i < numberOfValues) {
-    			final Object value = values[i];
+        String[] s = getStrings();
+        int numberOfValues = values.length;
+        for (int i = 0, size = s.length; i < size; i++) {
+            out.write(s[i]);
+            if (i < numberOfValues) {
+                final Object value = values[i];
 
-    			if (value instanceof Closure) {
-    				final Closure c = (Closure)value;
+                if (value instanceof Closure) {
+                    final Closure c = (Closure) value;
 
-    				if (c.getMaximumNumberOfParameters() == 0) {
-    					InvokerHelper.write(out, c.call(null));
-    				} else if (c.getMaximumNumberOfParameters() == 1) {
-    					c.call(new Object[]{out});
-    				} else {
-    					throw new GroovyRuntimeException("Trying to evaluate a GString containing a Closure taking "
-    							+ c.getMaximumNumberOfParameters() + " parameters");
-    				}
-    			} else {
-    				InvokerHelper.write(out, value);
-    			}
-    		}
-    	}
-    	return out;
+                    if (c.getMaximumNumberOfParameters() == 0) {
+                        InvokerHelper.write(out, c.call());
+                    } else if (c.getMaximumNumberOfParameters() == 1) {
+                        c.call(out);
+                    } else {
+                        throw new GroovyRuntimeException("Trying to evaluate a GString containing a Closure taking "
+                                + c.getMaximumNumberOfParameters() + " parameters");
+                    }
+                } else {
+                    InvokerHelper.write(out, value);
+                }
+            }
+        }
+        return out;
     }
 
     /* (non-Javadoc)
      * @see groovy.lang.Buildable#build(groovy.lang.GroovyObject)
      */
+
     public void build(final GroovyObject builder) {
-    final String[] s = getStrings();
-    final int numberOfValues = values.length;
+        final String[] s = getStrings();
+        final int numberOfValues = values.length;
 
         for (int i = 0, size = s.length; i < size; i++) {
             builder.getProperty("mkp");
@@ -246,6 +249,14 @@ public abstract class GString extends GroovyObjectSupport implements Comparable,
      * @return the regular expression pattern
      */
     public Pattern negate() {
-        return DefaultGroovyMethods.bitwiseNegate(toString());
+        return StringGroovyMethods.bitwiseNegate(toString());
+    }
+
+    public byte[] getBytes() {
+        return toString().getBytes();
+    }
+
+    public byte[] getBytes(String charset) throws UnsupportedEncodingException {
+       return toString().getBytes(charset);
     }
 }
