@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@ import org.codehaus.groovy.runtime.InvokerHelper;
  * any) at the same time.
  *
  * @author Scott Vlaminck (http://refactr.com)
- * @author Andres Almiray <aalmiray@users.sourceforge.com>
+ * @author <a href="mailto:aalmiray@users.sourceforge.com">Andres Almiray</a>
  */
 public class ObjectGraphBuilder extends FactoryBuilderSupport {
     public static final String NODE_CLASS = "_NODE_CLASS_";
@@ -51,7 +51,7 @@ public class ObjectGraphBuilder extends FactoryBuilderSupport {
     public static final String CLASSNAME_RESOLVER_REFLECTION_ROOT = "root";
 
     // Regular expression pattern used to identify words ending in 'y' preceded by a consonant
-    private static final Pattern pluralIESPattern = Pattern.compile(".*[^aeiouy]y", Pattern.CASE_INSENSITIVE);
+    private static final Pattern PLURAL_IES_PATTERN = Pattern.compile(".*[^aeiouy]y", Pattern.CASE_INSENSITIVE);
 
     private ChildPropertySetter childPropertySetter;
     private ClassNameResolver classNameResolver;
@@ -482,11 +482,11 @@ public class ObjectGraphBuilder extends FactoryBuilderSupport {
     }
 
     /**
-     * Default impl that returns parentName &amp; childName accordingly.
+     * Default impl that returns parentName and childName accordingly.
      */
     public static class DefaultRelationNameResolver implements RelationNameResolver {
         /**
-         * Handles the common English regular plurals with the following rules.<br />
+         * Handles the common English regular plurals with the following rules.
          * <ul>
          * <li>If childName ends in {consonant}y, replace 'y' with "ies". For example, allergy to allergies.</li>
          * <li>Otherwise, append 's'. For example, monkey to monkeys; employee to employees.</li>
@@ -497,7 +497,7 @@ public class ObjectGraphBuilder extends FactoryBuilderSupport {
          */
         public String resolveChildRelationName(String parentName, Object parent, String childName,
                                                Object child) {
-            boolean matchesIESRule = pluralIESPattern.matcher(childName).matches();
+            boolean matchesIESRule = PLURAL_IES_PATTERN.matcher(childName).matches();
             String childNamePlural = matchesIESRule ? childName.substring(0, childName.length() - 1) + "ies" : childName + "s";
 
             MetaProperty metaProperty = InvokerHelper.getMetaClass(parent)
