@@ -17,17 +17,21 @@
 package org.codehaus.groovy.ast;
 
 import groovy.lang.*;
+
 import org.codehaus.groovy.runtime.GeneratedClosure;
+import org.codehaus.groovy.util.ManagedConcurrentMap;
+import org.codehaus.groovy.util.ReferenceBundle;
 import org.codehaus.groovy.vmplugin.VMPluginFactory;
 import org.objectweb.asm.Opcodes;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.WeakHashMap;
 import java.util.regex.Pattern;
 import java.lang.ref.SoftReference;
+import java.lang.reflect.Modifier;
 
 /**
  * This class is a Helper for ClassNode and classes handling ClassNodes.
@@ -45,8 +49,9 @@ public class ClassHelper {
         Closure.class, GString.class, List.class, Map.class, Range.class,
         Pattern.class, Script.class, String.class,  Boolean.class, 
         Character.class, Byte.class, Short.class, Integer.class, Long.class,
-        Double.class, Float.class, BigDecimal.class, BigInteger.class, Void.class,
-        Reference.class, Class.class, MetaClass.class,     
+        Double.class, Float.class, BigDecimal.class, BigInteger.class, 
+        Number.class, Void.class, Reference.class, Class.class, MetaClass.class, 
+        Iterator.class, GeneratedClosure.class, GroovyObjectSupport.class
     };
 
     private static final String[] primitiveClassNames = new String[] {
@@ -58,7 +63,7 @@ public class ClassHelper {
         DYNAMIC_TYPE = makeCached(Object.class),  OBJECT_TYPE = DYNAMIC_TYPE,
         VOID_TYPE = makeCached(Void.TYPE),        CLOSURE_TYPE = makeCached(Closure.class),
         GSTRING_TYPE = makeCached(GString.class), LIST_TYPE = makeWithoutCaching(List.class),
-        MAP_TYPE = makeWithoutCaching(Map.class),         RANGE_TYPE = makeCached(Range.class),
+        MAP_TYPE = makeWithoutCaching(Map.class), RANGE_TYPE = makeCached(Range.class),
         PATTERN_TYPE = makeCached(Pattern.class), STRING_TYPE = makeCached(String.class),
         SCRIPT_TYPE = makeCached(Script.class),   REFERENCE_TYPE = makeWithoutCaching(Reference.class),
         
@@ -72,13 +77,23 @@ public class ClassHelper {
         Double_TYPE = makeCached(Double.class),       Boolean_TYPE = makeCached(Boolean.class),
         BigInteger_TYPE =  makeCached(java.math.BigInteger.class),
         BigDecimal_TYPE = makeCached(java.math.BigDecimal.class),
-        void_WRAPPER_TYPE = makeCached(Void.class),   
+        Number_TYPE = makeCached(Number.class),
+        
+        void_WRAPPER_TYPE = makeCached(Void.class),   METACLASS_TYPE = makeCached(MetaClass.class),
+        Iterator_TYPE = makeCached(Iterator.class),
+
+        // uncached constants.
+        CLASS_Type = makeWithoutCaching(Class.class), COMPARABLE_TYPE = makeWithoutCaching(Comparable.class),        
+        GENERATED_CLOSURE_Type = makeWithoutCaching(GeneratedClosure.class),
+        GROOVY_OBJECT_SUPPORT_TYPE = makeWithoutCaching(GroovyObjectSupport.class),
+        GROOVY_OBJECT_TYPE = makeWithoutCaching(GroovyObject.class),
+        GROOVY_INTERCEPTABLE_TYPE = makeWithoutCaching(GroovyInterceptable.class),
         
-        CLASS_Type = makeWithoutCaching(Class.class),        METACLASS_TYPE = makeCached(MetaClass.class),
-        GENERATED_CLOSURE_Type = makeCached(GeneratedClosure.class),
         Enum_Type = new ClassNode("java.lang.Enum",0,OBJECT_TYPE),
         Annotation_TYPE = new ClassNode("java.lang.annotation.Annotation",0,OBJECT_TYPE),
-        ELEMENT_TYPE_TYPE = new ClassNode("java.lang.annotation.ElementType",0,Enum_Type);
+        ELEMENT_TYPE_TYPE = new ClassNode("java.lang.annotation.ElementType",0,Enum_Type)
+        ;
+        
         
     static {
         Enum_Type.isPrimaryNode = false;
@@ -93,24 +108,21 @@ public class ClassHelper {
         LIST_TYPE, MAP_TYPE, RANGE_TYPE, PATTERN_TYPE,
         SCRIPT_TYPE, STRING_TYPE, Boolean_TYPE, Character_TYPE,
         Byte_TYPE, Short_TYPE, Integer_TYPE, Long_TYPE,
-        Double_TYPE, Float_TYPE, BigDecimal_TYPE, BigInteger_TYPE, 
+        Double_TYPE, Float_TYPE, BigDecimal_TYPE, BigInteger_TYPE,
+        Number_TYPE,
         void_WRAPPER_TYPE, REFERENCE_TYPE, CLASS_Type, METACLASS_TYPE,
-        GENERATED_CLOSURE_Type, Enum_Type, Annotation_TYPE
+        Iterator_TYPE, GENERATED_CLOSURE_Type, GROOVY_OBJECT_SUPPORT_TYPE, 
+        GROOVY_OBJECT_TYPE, GROOVY_INTERCEPTABLE_TYPE, Enum_Type, Annotation_TYPE
     };
 
+    private static final int ABSTRACT_STATIC_PRIVATE = 
+            Modifier.ABSTRACT|Modifier.PRIVATE|Modifier.STATIC;
+    private static final int VISIBILITY = 5; // public|protected
     
-    private static ClassNode[] numbers = new ClassNode[] {
-        char_TYPE, byte_TYPE, short_TYPE, int_TYPE, long_TYPE, 
-        double_TYPE, float_TYPE, Short_TYPE, Byte_TYPE, Character_TYPE,
-        Integer_TYPE, Float_TYPE, Long_TYPE, Double_TYPE, BigInteger_TYPE,
-        BigDecimal_TYPE
-    };
-
     protected static final ClassNode[] EMPTY_TYPE_ARRAY = {};
     
     public static final String OBJECT = "java.lang.Object";
 
-
     public static ClassNode makeCached(Class c){
         final SoftReference<ClassNode> classNodeSoftReference = ClassHelperCache.classCache.get(c);
         ClassNode classNode;
@@ -133,12 +145,12 @@ public class ClassHelper {
      * @return an array of ClassNodes
      */
     public static ClassNode[] make(Class[] classes) {
-    	ClassNode[] cns = new ClassNode[classes.length];
-    	for (int i=0; i<cns.length; i++) {
-    		cns[i] = make(classes[i]);
-    	}
-    	
-    	return cns;
+        ClassNode[] cns = new ClassNode[classes.length];
+        for (int i=0; i<cns.length; i++) {
+            cns[i] = make(classes[i]);
+        }
+        
+        return cns;
     }
     
     /**
@@ -260,7 +272,7 @@ public class ClassHelper {
         } else if (cn==double_TYPE) {
             return Double_TYPE;
         } else if (cn==VOID_TYPE) {
-        	return void_WRAPPER_TYPE;
+            return void_WRAPPER_TYPE;
         }
         else {
             return cn;
@@ -315,6 +327,30 @@ public class ClassHelper {
                 cn == VOID_TYPE;
     }
 
+    /**
+     * Test to determine if a ClassNode is a type belongs to the list of types which
+     * are allowed to initialize constants directly in bytecode instead of using &lt;cinit&gt;
+     *
+     * Note: this only works for ClassNodes created using a
+     * predefined ClassNode
+     *
+     * @see #make(Class)
+     * @see #make(String)
+     * @param cn the ClassNode to be tested
+     * @return true if the ClassNode is of int, float, long, double or String type
+     */
+    public static boolean isStaticConstantInitializerType(ClassNode cn) {
+        return  cn == int_TYPE ||
+                cn == float_TYPE ||
+                cn == long_TYPE ||
+                cn == double_TYPE ||
+                cn == STRING_TYPE ||
+                // the next items require conversion to int when initializing
+                cn == byte_TYPE ||
+                cn == char_TYPE ||
+                cn == short_TYPE;
+    }
+
     public static boolean isNumberType(ClassNode cn) {
         return  cn == Byte_TYPE ||
                 cn == Short_TYPE ||
@@ -331,17 +367,70 @@ public class ClassHelper {
     }
 
     public static ClassNode makeReference() {
-        return make(Reference.class);
+        return REFERENCE_TYPE.getPlainNodeReference();
     }
 
     public static boolean isCachedType(ClassNode type) {
-        for (int i=0;i<types.length; i++) {
-            if (types[i] == type) return true;
+        for (ClassNode cachedType : types) {
+            if (cachedType == type) return true;
         }
         return false;
     }
 
     static class ClassHelperCache {
-        static Map<Class, SoftReference<ClassNode>> classCache = new WeakHashMap<Class,SoftReference<ClassNode>>();
+        static ManagedConcurrentMap<Class, SoftReference<ClassNode>> classCache = new ManagedConcurrentMap<Class, SoftReference<ClassNode>>(ReferenceBundle.getWeakBundle());
+    }
+    
+    public static boolean isSAMType(ClassNode type) {
+        return findSAM(type) != null;
+    }
+
+    /**
+     * Returns the single abstract method of a class node, if it is a SAM type, or null otherwise.
+     * @param type a type for which to search for a single abstract method
+     * @return the method node if type is a SAM type, null otherwise
+     */
+    public static MethodNode findSAM(ClassNode type) {
+        if (!Modifier.isAbstract(type.getModifiers())) return null;
+        if (type.isInterface()) {
+            List<MethodNode> methods = type.getMethods();
+            MethodNode found=null;
+            for (MethodNode mi : methods) {
+                // ignore methods, that are not abstract and from Object
+                if (!Modifier.isAbstract(mi.getModifiers())) continue;
+                if (mi.getDeclaringClass().equals(OBJECT_TYPE)) continue;
+                if (OBJECT_TYPE.getDeclaredMethod(mi.getName(), mi.getParameters())!=null) continue;
+
+                // we have two methods, so no SAM
+                if (found!=null) return null;
+                found = mi;
+            }
+            return found;
+
+        } else {
+
+            List<MethodNode> methods = type.getAbstractMethods();
+            MethodNode found = null;
+            if (methods!=null) {
+                for (MethodNode mi : methods) {
+                    if (!hasUsableImplementation(type, mi)) {
+                        if (found!=null) return null;
+                        found = mi;
+                    }
+                }
+            }
+            return found;
+        }
+    }
+
+    private static boolean hasUsableImplementation(ClassNode c, MethodNode m) {
+        if (c==m.getDeclaringClass()) return false;
+        MethodNode found = c.getDeclaredMethod(m.getName(), m.getParameters());
+        if (found==null) return false;
+        int asp = found.getModifiers() & ABSTRACT_STATIC_PRIVATE;
+        int visible = found.getModifiers() & VISIBILITY;
+        if (visible !=0 && asp == 0) return true;
+        if (c.equals(OBJECT_TYPE)) return false;
+        return hasUsableImplementation(c.getSuperClass(), m);
     }
 }
