@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -33,10 +33,9 @@ import org.objectweb.asm.Opcodes;
 import java.lang.reflect.Array;
 import java.util.*;
 
-import groovy.lang.GroovyObject;
-
 /**
- * Represents a class in the AST.<br/>
+ * Represents a class in the AST.
+ * <p>
  * A ClassNode should be created using the methods in ClassHelper.
  * This ClassNode may be used to represent a class declaration or
  * any other type. This class uses a proxy mechanism allowing to
@@ -45,9 +44,8 @@ import groovy.lang.GroovyObject;
  * found. To avoid the need of exchanging this ClassNode with an
  * instance of the correct ClassNode the correct ClassNode is set as
  * redirect. Most method calls are then redirected to that ClassNode.
- * <br>
+ * <p>
  * There are three types of ClassNodes:
- * <br>
  * <ol>
  * <li> Primary ClassNodes:<br>
  * A primary ClassNode is one where we have a source representation
@@ -56,7 +54,6 @@ import groovy.lang.GroovyObject;
  * that passes through AsmBytecodeGenerator... not more, not less.
  * That means for example Closures become such ClassNodes too at
  * some point. 
- * 
  * <li> ClassNodes create through different sources (typically created
  * from a java.lang.reflect.Class object):<br>
  * The compiler will not output classes from these, the methods
@@ -70,7 +67,6 @@ import groovy.lang.GroovyObject;
  *  isResolved() returning true without having a redirect.In the Groovy 
  *  compiler the only version of this, that exists, is a ClassNode created 
  *  through a Class instance
- *
  * <li> Labels:<br>
  * ClassNodes created through ClassHelper.makeWithoutCaching. They 
  * are place holders, its redirect points to the real structure, which can
@@ -80,11 +76,11 @@ import groovy.lang.GroovyObject;
  * ResolveVisitor has done its work needs to have a redirect pointing to 
  * case 1 or 2. If not the compiler may react strange... this can be considered 
  * as a kind of dangling pointer. 
- * <br>
+ * </ol>
  * <b>Note:</b> the redirect mechanism is only allowed for classes 
  * that are not primary ClassNodes. Typically this is done for classes
  * created by name only.  The redirect itself can be any type of ClassNode.
- * <br>
+ * <p>
  * To describe generic type signature see {@link #getGenericsTypes()} and
  * {@link #setGenericsTypes(GenericsType[])}. These methods are not proxied,
  * they describe the type signature used at the point of declaration or the
@@ -92,10 +88,8 @@ import groovy.lang.GroovyObject;
  * by the class are needed, then a call to {@link #redirect()} will help.
  *
  * @see org.codehaus.groovy.ast.ClassHelper
- *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Jochen Theodorou
- * @version $Revision: 19496 $
  */
 public class ClassNode extends AnnotatedNode implements Opcodes {
     private static class MapOfLists {
@@ -125,6 +119,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     private String name;
     private int modifiers;
+    private boolean syntheticPublic;
     private ClassNode[] interfaces;
     private MixinNode[] mixins;
     private List<ConstructorNode> constructors;
@@ -214,8 +209,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     /**
      * @return true if this instance is a primary ClassNode
      */
-    public boolean isPrimaryClassNode(){
-    	return redirect().isPrimaryNode || (componentType!= null && componentType.isPrimaryClassNode());
+    public boolean isPrimaryClassNode() {
+        return redirect().isPrimaryNode || (componentType != null && componentType.isPrimaryClassNode());
     }
 
     /*
@@ -276,6 +271,22 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         redirect().enclosingMethod = enclosingMethod;
     }
 
+    /**
+     * Indicates that this class has been "promoted" to public by
+     * Groovy when in fact there was no public modifier explicitly
+     * in the source code. I.e. it remembers that it has applied
+     * Groovy's "public classes by default" rule.This property is
+     * typically only of interest to AST transform writers.
+     *
+     * @return true if this class is public but had no explicit public modifier
+     */
+    public boolean isSyntheticPublic() {
+        return syntheticPublic;
+    }
+
+    public void setSyntheticPublic(boolean syntheticPublic) {
+        this.syntheticPublic = syntheticPublic;
+    }
 
     /**
      * @param name       is the full name of the class
@@ -310,6 +321,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if (!usesGenerics && interfaces!=null) {
             for (ClassNode anInterface : interfaces) {
                 usesGenerics = usesGenerics || anInterface.isUsingGenerics();
+                if (usesGenerics) break;
             }
         }
         this.methods = new MapOfLists();
@@ -546,14 +558,14 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public boolean hasProperty(String name) {
-    	return getProperty(name)!=null;
+        return getProperty(name) != null;
     }
-    
+
     public PropertyNode getProperty(String name) {
-    	for (PropertyNode pn : getProperties()) {
+        for (PropertyNode pn : getProperties()) {
             if (pn.getName().equals(name)) return pn;
         }
-        return null;   	
+        return null;
     }
 
     public void addConstructor(ConstructorNode node) {
@@ -642,12 +654,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public void addInterface(ClassNode type) {
-        // lets check if it already implements an interface
+        // let's check if it already implements an interface
         boolean skip = false;
         ClassNode[] interfaces = redirect().interfaces;
-        for (int i = 0; i < interfaces.length; i++) {
-            if (type.equals(interfaces[i])) {
+        for (ClassNode existing : interfaces) {
+            if (type.equals(existing)) {
                 skip = true;
+                break;
             }
         }
         if (!skip) {
@@ -660,8 +673,9 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     public boolean equals(Object o) {
         if (redirect!=null) return redirect().equals(o);
+        if (!(o instanceof ClassNode)) return false;
         ClassNode cn = (ClassNode) o;
-        return (cn.getName().equals(getName()));
+        return (cn.getText().equals(getText()));
     }
 
     public int hashCode() {
@@ -670,12 +684,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public void addMixin(MixinNode mixin) {
-        // lets check if it already uses a mixin
+        // let's check if it already uses a mixin
         MixinNode[] mixins = redirect().mixins;
         boolean skip = false;
-        for (int i = 0; i < mixins.length; i++) {
-            if (mixin.equals(mixins[i])) {
+        for (MixinNode existing : mixins) {
+            if (mixin.equals(existing)) {
                 skip = true;
+                break;
             }
         }
         if (!skip) {
@@ -693,6 +708,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return the method matching the given name and parameters or null
      */
     public FieldNode getDeclaredField(String name) {
+        if (!redirect().lazyInitDone) redirect().lazyClassInit();
         ClassNode r = redirect ();
         if (r.fieldIndex == null)
             r.fieldIndex = new HashMap<String,FieldNode> ();
@@ -741,7 +757,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     public List<Statement> getObjectInitializerStatements() {
         if (objectInitializers == null)
-            objectInitializers = new ArrayList<Statement> ();
+            objectInitializers = new LinkedList<Statement> ();
         return objectInitializers;
     }
 
@@ -792,29 +808,29 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         MethodNode method = getOrAddStaticConstructorNode();
         Statement statement = method.getCode();
         if (statement instanceof BlockStatement) {
-        	BlockStatement block = (BlockStatement) statement;
-        	// add given statements for explicitly declared static fields just after enum-special fields
-        	// are found - the $VALUES binary expression marks the end of such fields.
-        	List<Statement> blockStatements = block.getStatements();
-        	ListIterator<Statement> litr = blockStatements.listIterator();
-        	while(litr.hasNext()) {
-        		Statement stmt = litr.next();
-        		if(stmt instanceof ExpressionStatement && 
-        				((ExpressionStatement)stmt).getExpression() instanceof BinaryExpression) {
-        			BinaryExpression bExp = (BinaryExpression) ((ExpressionStatement)stmt).getExpression();
-        			if (bExp.getLeftExpression() instanceof FieldExpression) {
-        				FieldExpression fExp = (FieldExpression) bExp.getLeftExpression();
-        				if(fExp.getFieldName().equals("$VALUES")) {
-        					for(Statement tmpStmt : staticFieldStatements) {
-        						litr.add(tmpStmt);
-        					}
-        				}
-        			}
-        		}
-        	}
+            BlockStatement block = (BlockStatement) statement;
+            // add given statements for explicitly declared static fields just after enum-special fields
+            // are found - the $VALUES binary expression marks the end of such fields.
+            List<Statement> blockStatements = block.getStatements();
+            ListIterator<Statement> litr = blockStatements.listIterator();
+            while (litr.hasNext()) {
+                Statement stmt = litr.next();
+                if (stmt instanceof ExpressionStatement &&
+                        ((ExpressionStatement) stmt).getExpression() instanceof BinaryExpression) {
+                    BinaryExpression bExp = (BinaryExpression) ((ExpressionStatement) stmt).getExpression();
+                    if (bExp.getLeftExpression() instanceof FieldExpression) {
+                        FieldExpression fExp = (FieldExpression) bExp.getLeftExpression();
+                        if (fExp.getFieldName().equals("$VALUES")) {
+                            for (Statement tmpStmt : staticFieldStatements) {
+                                litr.add(tmpStmt);
+                            }
+                        }
+                    }
+                }
+            }
         }
     }
-    
+
     /**
      * This methods returns a list of all methods of the given name
      * defined in the current class
@@ -877,9 +893,9 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return true if this node is derived from the given ClassNode
      */
     public boolean isDerivedFrom(ClassNode type) {
-    	if(this.equals(ClassHelper.VOID_TYPE)) {
-    		return type.equals(ClassHelper.VOID_TYPE) ? true : false;
-    	}
+        if (this.equals(ClassHelper.VOID_TYPE)) {
+            return type.equals(ClassHelper.VOID_TYPE);
+        }
         if (type.equals(ClassHelper.OBJECT_TYPE)) return true;
         ClassNode node = this;
         while (node != null) {
@@ -896,7 +912,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      *         i.e. it implements GroovyObject
      */
     public boolean isDerivedFromGroovyObject() {
-        return implementsInterface(ClassHelper.make(GroovyObject.class));
+        return implementsInterface(ClassHelper.GROOVY_OBJECT_TYPE);
     }
 
     /**
@@ -1054,20 +1070,24 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public MethodNode getSetterMethod(String setterName) {
+        return getSetterMethod(setterName, true);
+    }
+
+    public MethodNode getSetterMethod(String setterName, boolean voidOnly) {
         for (MethodNode method : getDeclaredMethods(setterName)) {
             if (setterName.equals(method.getName())
-                    && ClassHelper.VOID_TYPE==method.getReturnType()
+                    && (!voidOnly || ClassHelper.VOID_TYPE==method.getReturnType())
                     && method.getParameters().length == 1) {
                 return method;
             }
         }
         ClassNode parent = getSuperClass();
-        if (parent!=null) return parent.getSetterMethod(setterName);
+        if (parent!=null) return parent.getSetterMethod(setterName, voidOnly);
         return null;
     }
 
     /**
-     * Is this class delcared in a static method (such as a closure / inner class declared in a static method)
+     * Is this class declared in a static method (such as a closure / inner class declared in a static method)
      */
     public boolean isStaticClass() {
         return redirect().staticClass;
@@ -1097,21 +1117,61 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public String toString() {
+        return toString(true);
+    }
+
+    public String toString(boolean showRedirect) {
+        if (isArray()) {
+            return componentType.toString(showRedirect)+"[]";
+        }
         String ret = getName();
-        if (genericsTypes != null) {
+        if (placeholder) ret = getUnresolvedName();
+        if (!placeholder && genericsTypes != null) {
             ret += " <";
             for (int i = 0; i < genericsTypes.length; i++) {
                 if (i != 0) ret += ", ";
-                ret += genericsTypes[i];
+                GenericsType genericsType = genericsTypes[i];
+                ret += genericTypeAsString(genericsType, showRedirect);
             }
             ret += ">";
         }
-        if (redirect != null) {
+        if (redirect != null && showRedirect) {
             ret += " -> " + redirect().toString();
         }
         return ret;
     }
 
+    /**
+     * This exists to avoid a recursive definition of toString. The default toString
+     * in GenericsType calls ClassNode.toString(), which calls GenericsType.toString(), etc. 
+     * @param genericsType
+     * @param showRedirect
+     * @return the string representing the generic type
+     */
+    private String genericTypeAsString(GenericsType genericsType, boolean showRedirect) {
+        String ret = genericsType.getName();
+        if (genericsType.getUpperBounds() != null) {
+            ret += " extends ";
+            for (int i = 0; i < genericsType.getUpperBounds().length; i++) {
+                ClassNode classNode = genericsType.getUpperBounds()[i];
+                if (classNode.equals(this)) {
+                    ret += classNode.getName();
+                } else {
+                    ret += classNode.toString(showRedirect);
+                }
+                if (i + 1 < genericsType.getUpperBounds().length) ret += " & ";
+            }
+        } else if (genericsType.getLowerBound() !=null) {
+            ClassNode classNode = genericsType.getLowerBound();
+            if (classNode.equals(this)) {
+                ret += " super " + classNode.getName();
+            } else {
+                ret += " super " + classNode;
+            }
+        }
+        return ret;
+    }
+
     /**
      * Returns true if the given method has a possibly matching instance method with the given name and arguments.
      *
@@ -1206,7 +1266,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
             // TODO this won't strictly be true when using list expansion in argument calls
             count = tuple.getExpressions().size();
         } else if (arguments instanceof MapExpression) {
-        	count = 1;
+            count = 1;
         }
         
         for (MethodNode method : getMethods(name)) {
@@ -1221,15 +1281,15 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
                 
                 // handle parameters with default values
                 int nonDefaultParameters = 0;
-                for(int i = 0; i < parameters.length; i++) {
-                	if(parameters[i].hasInitialExpression() == false) {
-                		nonDefaultParameters++;
-                	}
+                for (Parameter parameter : parameters) {
+                    if (!parameter.hasInitialExpression()) {
+                        nonDefaultParameters++;
+                    }
+                }
+
+                if (count < parameters.length && nonDefaultParameters <= count) {
+                    return true;
                 }
-                
-            	if(count < parameters.length && nonDefaultParameters <= count) {
-            		return true;
-            	}
             }
         }
         return false;
@@ -1251,6 +1311,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return componentType;
     }
 
+    /**
+     * Returns the concrete class this classnode relates to. However, this method
+     * is inherently unsafe as it may return null depending on the compile phase you are
+     * using. AST transformations should never use this method directly, but rather obtain
+     * a new class node using {@link #getPlainNodeReference()}.
+     * @return the class this classnode relates to. May return null.
+     */
     public Class getTypeClass(){
         Class c = redirect().clazz;
         if (c!=null) return c;
@@ -1307,9 +1374,10 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     public ClassNode getPlainNodeReference() {
         if (ClassHelper.isPrimitiveType(this)) return this;
-        ClassNode n = new ClassNode(getName(),getModifiers(),getSuperClass(),null,null);
+        ClassNode n = new ClassNode(name, modifiers, superClass,null,null);
         n.isPrimaryNode = false;
-        n.setRedirect(this.redirect);
+        n.setRedirect(redirect());
+        n.componentType = redirect().getComponentType();
         return n;
     }
 
@@ -1333,6 +1401,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     public void addTransform(Class<? extends ASTTransformation> transform, ASTNode node) {
         GroovyASTTransformation annotation = transform.getAnnotation(GroovyASTTransformation.class);
+        if (annotation == null) return;
+
         Set<ASTNode> nodes = getTransformInstances().get(annotation.phase()).get(transform);
         if (nodes == null) {
             nodes = new LinkedHashSet<ASTNode>();
@@ -1353,6 +1423,15 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         index.put(newName, index.remove(oldName));
     }
     
+    public void removeField(String oldName) {
+        ClassNode r = redirect ();
+        if (r.fieldIndex == null)
+            r.fieldIndex = new HashMap<String,FieldNode> ();
+        final Map<String,FieldNode> index = r.fieldIndex;
+        r.fields.remove(index.get(oldName));
+        index.remove(oldName);
+    }
+
     public boolean isEnum() {
         return (getModifiers()&Opcodes.ACC_ENUM) != 0;
      }
@@ -1373,4 +1452,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         }
         return transformInstances;
     }
+    
+    public boolean isRedirectNode() {
+        return redirect!=null;
+    }
+
+    @Override
+    public String getText() {
+        return getName();
+    }
 }
