@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -19,9 +19,9 @@ import org.objectweb.asm.Opcodes;
 
 /**
  * Represents an import statement of a single class
- * 
- * author Jochen Theodorou
- * author Paul King
+ *
+ * @author Jochen Theodorou
+ * @author Paul King
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  */
 public class ImportNode extends AnnotatedNode implements Opcodes {
@@ -35,9 +35,9 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
     private final boolean isStatic;
 
     /**
-     * Represent an import of an entire package, i.e. import package.Classname
+     * Represent an import of an entire package, i.e.&#160;import package.Classname
      *
-     * @param type the referenced class
+     * @param type  the referenced class
      * @param alias optional alias
      */
     public ImportNode(ClassNode type, String alias) {
@@ -50,7 +50,7 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
-     * Represent an import of an entire package, i.e. import package.*
+     * Represent an import of an entire package, i.e.&#160;import package.*
      *
      * @param packageName the name of the package
      */
@@ -64,7 +64,7 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
-     * Represent a static import of a Class, i.e. import static package.Classname.*
+     * Represent a static import of a Class, i.e.&#160;import static package.Classname.*
      *
      * @param type the referenced class
      */
@@ -72,17 +72,17 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
         this.type = type;
         this.alias = null;
         this.isStar = true;
-        this.isStatic = false;
+        this.isStatic = true;
         this.packageName = null;
         this.fieldName = null;
     }
 
     /**
-     * Represent a static import of a field or method, i.e. import static package.Classname.name
+     * Represent a static import of a field or method, i.e.&#160;import static package.Classname.name
      *
-     * @param type the referenced class
+     * @param type      the referenced class
      * @param fieldName the field name
-     * @param alias optional alias
+     * @param alias     optional alias
      */
     public ImportNode(ClassNode type, String fieldName, String alias) {
         this.type = type;
@@ -97,7 +97,7 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
      * @return the text display of this import
      */
     public String getText() {
-    	String typeName = getClassName();
+        String typeName = getClassName();
         if (isStar && !isStatic) {
             return "import " + packageName + "*";
         }
@@ -115,7 +115,7 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
         }
         return "import " + typeName + " as " + alias;
     }
-    
+
     public String getPackageName() {
         return packageName;
     }
@@ -139,9 +139,9 @@ public class ImportNode extends AnnotatedNode implements Opcodes {
     public ClassNode getType() {
         return type;
     }
-    
+
     public String getClassName() {
-    	return type == null ? null : type.getName();
+        return type == null ? null : type.getName();
     }
 
     public void visit(GroovyCodeVisitor visitor) {
