@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2011 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,8 +23,10 @@ import org.codehaus.groovy.ast.expr.VariableExpression;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
+import org.codehaus.groovy.control.ErrorCollector;
 import org.codehaus.groovy.control.SourceUnit;
 import org.codehaus.groovy.runtime.InvokerHelper;
+import org.codehaus.groovy.syntax.SyntaxException;
 import org.objectweb.asm.Opcodes;
 
 import java.io.File;
@@ -55,6 +57,7 @@ public class ModuleNode extends ASTNode implements Opcodes {
     private transient SourceUnit context;
     private boolean importsResolved = false;
     private ClassNode scriptDummy;
+    private String mainClassName = null;
 
     public ModuleNode (SourceUnit context ) {
         this.context = context;
@@ -75,6 +78,7 @@ public class ModuleNode extends ASTNode implements Opcodes {
     public List<ClassNode> getClasses() {
         if (createClassForStatements && (!statementBlock.isEmpty() || !methods.isEmpty() || isPackageInfo())) {
             ClassNode mainClass = createStatementsClass();
+            mainClassName = mainClass.getName(); 
             createClassForStatements = false;
             classes.add(0, mainClass);
             mainClass.setModule(this);
@@ -91,18 +95,6 @@ public class ModuleNode extends ASTNode implements Opcodes {
         return new ArrayList<ImportNode>(imports.values());
     }
 
-    /**
-     * @deprecated replaced by {@link #getStarImports()}
-     */
-    @Deprecated
-    public List<String> getImportPackages() {
-        List<String> result = new ArrayList<String>();
-        for (ImportNode importStarNode : starImports) {
-            result.add(importStarNode.getPackageName());
-        }
-        return result;
-    }
-
     public List<ImportNode> getStarImports() {
         return starImports;
     }
@@ -132,15 +124,7 @@ public class ModuleNode extends ASTNode implements Opcodes {
         ImportNode importNode = new ImportNode(type, alias);
         imports.put(alias, importNode);
         importNode.addAnnotations(annotations);
-    }
-
-    /**
-     * @deprecated replaced by {@link #addStarImport(String)}
-     */
-    @Deprecated
-    public String[] addImportPackage(String packageName) {
-        addStarImport(packageName);
-        return new String[]{};
+        storeLastAddedImportNode(importNode);
     }
 
     public void addStarImport(String packageName) {
@@ -151,6 +135,7 @@ public class ModuleNode extends ASTNode implements Opcodes {
         ImportNode importNode = new ImportNode(packageName);
         importNode.addAnnotations(annotations);
         starImports.add(importNode);
+        storeLastAddedImportNode(importNode);
     }
 
     public void addStatement(Statement node) {
@@ -158,6 +143,7 @@ public class ModuleNode extends ASTNode implements Opcodes {
     }
 
     public void addClass(ClassNode node) {
+        if(classes.isEmpty()) mainClassName = node.getName();
         classes.add(node);
         node.setModule(this);
         addToCompileUnit(node);
@@ -222,8 +208,6 @@ public class ModuleNode extends ASTNode implements Opcodes {
     }
 
     public void setDescription(String description) {
-        // DEPRECATED -- context.getName() is now sufficient
-        // TODO add deprecated annotation or javadoc comment?
         this.description = description;
     }
 
@@ -236,7 +220,10 @@ public class ModuleNode extends ASTNode implements Opcodes {
     }
 
     public ClassNode getScriptClassDummy() {
-        if (scriptDummy!=null) return scriptDummy;
+        if (scriptDummy!=null) {
+            setScriptBaseClassFromConfig(scriptDummy);
+            return scriptDummy;
+        }
         
         String name = getPackageName();
         if (name == null) {
@@ -248,20 +235,12 @@ public class ModuleNode extends ASTNode implements Opcodes {
         }
         name += extractClassFromFileDescription();
 
-        String baseClassName = null;
-        if (unit != null) baseClassName = unit.getConfig().getScriptBaseClass();
-        ClassNode baseClass = null;
-        if (baseClassName!=null) {
-            baseClass = ClassHelper.make(baseClassName);
-        }
-        if (baseClass == null) {
-            baseClass = ClassHelper.SCRIPT_TYPE;
-        }
         ClassNode classNode;
         if (isPackageInfo()) {
             classNode = new ClassNode(name, ACC_ABSTRACT | ACC_INTERFACE, ClassHelper.OBJECT_TYPE);
         } else {
-            classNode = new ClassNode(name, ACC_PUBLIC, baseClass);
+            classNode = new ClassNode(name, ACC_PUBLIC, ClassHelper.SCRIPT_TYPE);
+            setScriptBaseClassFromConfig(classNode);
             classNode.setScript(true);
             classNode.setScriptBody(true);
         }
@@ -269,6 +248,20 @@ public class ModuleNode extends ASTNode implements Opcodes {
         scriptDummy = classNode;
         return classNode;
     }
+
+    private void setScriptBaseClassFromConfig(ClassNode cn) {
+        String baseClassName = null;
+        if (unit != null) {
+            baseClassName = unit.getConfig().getScriptBaseClass();
+        } else if (context != null) {
+            baseClassName = context.getConfiguration().getScriptBaseClass();
+        }
+        if (baseClassName != null) {
+            if (!cn.getSuperClass().getName().equals(baseClassName)) {
+                cn.setSuperClass(ClassHelper.make(baseClassName));
+            }
+        }
+    }
     
     protected ClassNode createStatementsClass() {
         ClassNode classNode = getScriptClassDummy();
@@ -294,21 +287,29 @@ public class ModuleNode extends ASTNode implements Opcodes {
                                 new ClassExpression(classNode),
                                 new VariableExpression("args"))))));
 
-        classNode.addMethod(
-            new MethodNode("run", ACC_PUBLIC, ClassHelper.OBJECT_TYPE, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, statementBlock));
-
+        MethodNode methodNode = hasRunMethod();
+        if (methodNode!=null) {
+            ErrorCollector ec = context.getErrorCollector();
+            ec.addError(new SyntaxException("You cannot define a 'run()' method in a script because it is used to wrap the script body. Please choose another name.",
+                        methodNode.getLineNumber(),
+                        methodNode.getLineNumber()),
+                    context);
+        } else {
+            classNode.addMethod(
+                    new MethodNode("run", ACC_PUBLIC, ClassHelper.OBJECT_TYPE, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, statementBlock));
+        }
         classNode.addConstructor(ACC_PUBLIC, Parameter.EMPTY_ARRAY, ClassNode.EMPTY_ARRAY, new BlockStatement());
         Statement stmt = new ExpressionStatement(
                         new MethodCallExpression(
                             new VariableExpression("super"),
-            				"setBinding",
-            				new ArgumentListExpression(
+                            "setBinding",
+                            new ArgumentListExpression(
                                         new VariableExpression("context"))));
 
         classNode.addConstructor(
             ACC_PUBLIC,
             new Parameter[] { new Parameter(ClassHelper.make(Binding.class), "context")},
-			ClassNode.EMPTY_ARRAY,
+            ClassNode.EMPTY_ARRAY,
             stmt);
 
         for (MethodNode node : methods) {
@@ -327,15 +328,25 @@ public class ModuleNode extends ASTNode implements Opcodes {
         return classNode;
     }
 
+    private MethodNode hasRunMethod() {
+        MethodNode methodNode = null;
+        for (MethodNode method : methods) {
+            if ("run".equals(method.getName()) && method.getParameters().length==0) {
+                methodNode = method;
+                break;
+            }
+        }
+        return methodNode;
+    }
+
     /*
      * If a main method is provided by user, account for it under run() as scripts generate their own 'main' so they can run.  
      */
     private void handleMainMethodIfPresent(List methods) {
-    	boolean found = false;
+        boolean found = false;
         for (Iterator iter = methods.iterator(); iter.hasNext();) {
             MethodNode node = (MethodNode) iter.next();
             if(node.getName().equals("main")) {
-                int modifiers = node.getModifiers();
                 if (node.isStatic() && node.getParameters().length == 1) {
                     boolean retTypeMatches, argTypeMatches;
                     ClassNode argType = node.getParameters()[0].getType();
@@ -345,11 +356,11 @@ public class ModuleNode extends ASTNode implements Opcodes {
                     retTypeMatches = (retType == ClassHelper.VOID_TYPE || retType == ClassHelper.OBJECT_TYPE);
                     
                     if(retTypeMatches && argTypeMatches) {
-                    	if(found) {
-                    		throw new RuntimeException("Repetitive main method found.");
-                    	} else {
-                    		found = true;
-                    	}
+                        if(found) {
+                            throw new RuntimeException("Repetitive main method found.");
+                        } else {
+                            found = true;
+                        }
                         // if script has both loose statements as well as main(), then main() is ignored
                         if(statementBlock.isEmpty()) {
                             addStatement(node.getCode());
@@ -387,22 +398,22 @@ public class ModuleNode extends ASTNode implements Opcodes {
     }
     
     public void sortClasses(){
-    	if (isEmpty()) return;
-    	List<ClassNode> classes = getClasses();
-    	LinkedList<ClassNode> sorted = new LinkedList<ClassNode>();
-    	int level=1;
-    	while (!classes.isEmpty()) {
-	    	for (Iterator<ClassNode> cni = classes.iterator(); cni.hasNext();) {
-				ClassNode cn = cni.next();
-				ClassNode sn = cn;
-				for (int i=0; sn!=null && i<level; i++) sn = sn.getSuperClass();
-				if (sn!=null && sn.isPrimaryClassNode()) continue;
-				cni.remove();
-				sorted.addLast(cn);
-			}
-	    	level++;
-    	}
-    	this.classes = sorted;
+        if (isEmpty()) return;
+        List<ClassNode> classes = getClasses();
+        LinkedList<ClassNode> sorted = new LinkedList<ClassNode>();
+        int level=1;
+        while (!classes.isEmpty()) {
+            for (Iterator<ClassNode> cni = classes.iterator(); cni.hasNext();) {
+                ClassNode cn = cni.next();
+                ClassNode sn = cn;
+                for (int i=0; sn!=null && i<level; i++) sn = sn.getSuperClass();
+                if (sn!=null && sn.isPrimaryClassNode()) continue;
+                cni.remove();
+                sorted.addLast(cn);
+            }
+            level++;
+        }
+        this.classes = sorted;
     }
 
     public boolean hasImportsResolved() {
@@ -413,42 +424,6 @@ public class ModuleNode extends ASTNode implements Opcodes {
         this.importsResolved = importsResolved;
     }
 
-    /**
-     * @deprecated replaced by {@link #getStaticImports()}
-     */
-    @Deprecated
-    public Map<String, ClassNode> getStaticImportAliases() {
-        Map<String, ClassNode> result = new HashMap<String, ClassNode>();
-        for (Map.Entry<String, ImportNode> entry : staticImports.entrySet()) {
-            result.put(entry.getKey(), entry.getValue().getType());
-        }
-        return result;
-    }
-
-    /**
-     * @deprecated replaced by {@link #getStaticStarImports()}
-     */
-    @Deprecated
-    public Map<String, ClassNode> getStaticImportClasses() {
-        Map<String, ClassNode> result = new HashMap<String, ClassNode>();
-        for (Map.Entry<String, ImportNode> entry : staticStarImports.entrySet()) {
-            result.put(entry.getKey(), entry.getValue().getType());
-        }
-        return result;
-    }
-
-    /**
-     * @deprecated replaced by {@link #getStaticImports()}
-     */
-    @Deprecated
-    public Map<String, String> getStaticImportFields() {
-        Map<String, String> result = new HashMap<String, String>();
-        for (Map.Entry<String, ImportNode> entry : staticImports.entrySet()) {
-            result.put(entry.getKey(), entry.getValue().getFieldName());
-        }
-        return result;
-    }
-
     public Map<String, ImportNode> getStaticImports() {
         return staticImports;
     }
@@ -457,14 +432,6 @@ public class ModuleNode extends ASTNode implements Opcodes {
         return staticStarImports;
     }
 
-    /**
-     * @deprecated replaced by {@link #addStaticImport(ClassNode, String, String)}
-     */
-    @Deprecated
-    public void addStaticMethodOrField(ClassNode type, String fieldName, String alias) {
-        addStaticImport(type, fieldName, alias);
-    }
-
     public void addStaticImport(ClassNode type, String fieldName, String alias) {
         addStaticImport(type, fieldName, alias, new ArrayList<AnnotationNode>());
     }
@@ -473,14 +440,7 @@ public class ModuleNode extends ASTNode implements Opcodes {
         ImportNode node = new ImportNode(type, fieldName, alias);
         node.addAnnotations(annotations);
         staticImports.put(alias, node);
-    }
-
-    /**
-     * @deprecated replaced by {@link #addStaticStarImport(String, ClassNode)}
-     */
-    @Deprecated
-    public void addStaticImportClass(String name, ClassNode type) {
-        addStaticStarImport(name, type);
+        storeLastAddedImportNode(node);
     }
 
     public void addStaticStarImport(String name, ClassNode type) {
@@ -491,5 +451,18 @@ public class ModuleNode extends ASTNode implements Opcodes {
         ImportNode node = new ImportNode(type);
         node.addAnnotations(annotations);
         staticStarImports.put(name, node);
+        storeLastAddedImportNode(node);
+    }
+
+    // This method only exists as a workaround for GROOVY-6094
+    // In order to keep binary compatibility
+    private void storeLastAddedImportNode(final ImportNode node) {
+        if (getNodeMetaData(ImportNode.class)==ImportNode.class) {
+            putNodeMetaData(ImportNode.class, node);
+        }
+    }
+
+    public String getMainClassName() {
+        return mainClassName;
     }
 }
