@@ -23,7 +23,7 @@ import org.codehaus.groovy.ast.GroovyCodeVisitor;
 import org.codehaus.groovy.ast.VariableScope;
 
 /**
- * This class rerpresents a list of expressions used to 
+ * This class represents a list of expressions used to
  * create closures. Example:
  * <code>
  * def foo = (1;2;;)
@@ -56,6 +56,7 @@ public class ClosureListExpression extends ListExpression {
     public Expression transformExpression(ExpressionTransformer transformer) {
         Expression ret = new ClosureListExpression(transformExpressions(getExpressions(), transformer));
         ret.setSourcePosition(this);
+        ret.copyNodeMetaData(this);
         return ret;       
     }
     
@@ -68,7 +69,7 @@ public class ClosureListExpression extends ListExpression {
     }
     
     public String getText() {
-        StringBuffer buffer = new StringBuffer("(");
+        StringBuilder buffer = new StringBuilder("(");
         boolean first = true;
         for (Expression expression : getExpressions()) {
             if (first) {
