@@ -26,9 +26,9 @@ import org.codehaus.groovy.ast.expr.Expression;
  * Represents a standard for loop in Groovy
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 15760 $
+ * @version $Revision$
  */
-public class ForStatement extends Statement {
+public class ForStatement extends Statement implements LoopingStatement {
     public static final Parameter FOR_LOOP_DUMMY = new Parameter(ClassHelper.OBJECT_TYPE,"forLoopDummyParameter");
 
     private Parameter variable;
