@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2009 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,18 +17,22 @@ package org.codehaus.groovy.classgen;
 
 import org.codehaus.groovy.GroovyBugError;
 import org.codehaus.groovy.ast.*;
-import org.codehaus.groovy.ast.Variable;
 import org.codehaus.groovy.ast.expr.*;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.CatchStatement;
 import org.codehaus.groovy.ast.stmt.ForStatement;
+import org.codehaus.groovy.ast.stmt.IfStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.control.SourceUnit;
+import org.codehaus.groovy.syntax.Types;
 
+import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 
+import static java.lang.reflect.Modifier.isFinal;
+
 /**
  * goes through an AST and initializes the scopes
  *
@@ -40,8 +44,6 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     private VariableScope headScope = new VariableScope();
     private ClassNode currentClass = null;
     private SourceUnit source;
-    private boolean inClosure = false;
-    private boolean inPropertyExpression = false;
     private boolean isSpecialConstructorCall = false;
     private boolean inConstructor = false;
 
@@ -50,13 +52,11 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     private class StateStackElement {
         VariableScope scope;
         ClassNode clazz;
-        boolean closure;
-        boolean inConstructor; 
+        boolean inConstructor;
 
         StateStackElement() {
             scope = VariableScopeVisitor.this.currentScope;
             clazz = VariableScopeVisitor.this.currentClass;
-            closure = VariableScopeVisitor.this.inClosure;
             inConstructor = VariableScopeVisitor.this.inConstructor;
         }
     }
@@ -81,18 +81,9 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     private void popState() {
-        // a scope in a closure is never really static
-        // the checking needs this to be as the surrounding
-        // method to correctly check the access to variables.
-        // But a closure and all nested scopes are a result
-        // of calling a non static method, so the context
-        // is not static.
-        if (inClosure) currentScope.setInStaticContext(false);
-
         StateStackElement element = (StateStackElement) stateStack.removeLast();
         currentScope = element.scope;
         currentClass = element.clazz;
-        inClosure = element.closure;
         inConstructor = element.inConstructor;
     }
 
@@ -123,7 +114,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
             variableType = "property";
         }
 
-        StringBuffer msg = new StringBuffer();
+        StringBuilder msg = new StringBuilder();
         msg.append("The current ").append(scopeType);
         msg.append(" already contains a ").append(variableType);
         msg.append(" of the name ").append(var.getName());
@@ -240,7 +231,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
                     boolean staticMember = member.isInStaticContext();
                     // We don't allow a static context (e.g. a static method) to access
                     // a non-static variable (e.g. a non-static field).
-                    if (! (staticScope && ! staticMember))
+                    if (!(staticScope && !staticMember))
                         var = member;
                 }
                 break;
@@ -256,7 +247,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
                     (end.isReferencedClassVariable(name) && end.getDeclaredVariable(name) == null)) {
                 scope.putReferencedClassVariable(var);
             } else {
-                var.setClosureSharedVariable(var.isClosureSharedVariable() || inClosure);
+                //var.setClosureSharedVariable(var.isClosureSharedVariable() || inClosure);
                 scope.putReferencedLocalVariable(var);
             }
             scope = scope.getParent();
@@ -279,15 +270,14 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         VariableExpression ve = (VariableExpression) object;
         if (!ve.getName().equals("this")) return;
         String name = pe.getPropertyAsString();
-        if (name.equals("class")) return;
-        if (name == null) return;
+        if (name == null || name.equals("class")) return;
         Variable member = findClassMember(currentClass, name);
         if (member == null) return;
         checkVariableContextAccess(member, pe);
     }
 
     private void checkVariableContextAccess(Variable v, Expression expr) {
-        if (inPropertyExpression || v.isInStaticContext() || !currentScope.isInStaticContext()) return;
+        if (v.isInStaticContext() || !currentScope.isInStaticContext()) return;
 
         String msg = v.getName() +
                 " is declared in a dynamic context, but you tried to" +
@@ -320,23 +310,79 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         popState();
     }
 
+    public void visitIfElse(IfStatement ifElse) {
+        ifElse.getBooleanExpression().visit(this);
+        pushState();
+        ifElse.getIfBlock().visit(this);
+        popState();
+        pushState();
+        ifElse.getElseBlock().visit(this);
+        popState();
+    }
+    
     public void visitDeclarationExpression(DeclarationExpression expression) {
         // visit right side first to avoid the usage of a
         // variable before its declaration
         expression.getRightExpression().visit(this);
 
-        // no need to visit left side, just get the variable name
         if (expression.isMultipleAssignmentDeclaration()) {
-            ArgumentListExpression list = (ArgumentListExpression) expression.getLeftExpression();
+            TupleExpression list = expression.getTupleExpression();
             for (Expression e : list.getExpressions()) {
-                VariableExpression exp = (VariableExpression) e;
-                declare(exp);
+                declare((VariableExpression) e);
             }
         } else {
             declare(expression.getVariableExpression());
         }
     }
 
+    @Override
+    public void visitBinaryExpression(BinaryExpression be) {
+        super.visitBinaryExpression(be);
+        switch (be.getOperation().getType()) {
+            case Types.EQUAL: // = assignment
+            case Types.BITWISE_AND_EQUAL:
+            case Types.BITWISE_OR_EQUAL:
+            case Types.BITWISE_XOR_EQUAL:
+            case Types.PLUS_EQUAL:
+            case Types.MINUS_EQUAL:
+            case Types.MULTIPLY_EQUAL:
+            case Types.DIVIDE_EQUAL:
+            case Types.INTDIV_EQUAL:
+            case Types.MOD_EQUAL:
+            case Types.POWER_EQUAL:
+            case Types.LEFT_SHIFT_EQUAL:
+            case Types.RIGHT_SHIFT_EQUAL:
+            case Types.RIGHT_SHIFT_UNSIGNED_EQUAL:
+                checkFinalFieldAccess(be.getLeftExpression());
+                break;
+            default:
+                break;
+        }
+    }
+
+    private void checkFinalFieldAccess(Expression expression) {
+        // currently not looking for PropertyExpression: dealt with at runtime using ReadOnlyPropertyException
+        if (!(expression instanceof VariableExpression) && !(expression instanceof TupleExpression)) return;
+        if (expression instanceof TupleExpression) {
+            TupleExpression list = (TupleExpression) expression;
+            for (Expression e : list.getExpressions()) {
+                checkForFinal(expression, (VariableExpression) e);
+            }
+        } else {
+            checkForFinal(expression, (VariableExpression) expression);
+        }
+    }
+
+    // TODO handle local variables
+    private void checkForFinal(final Expression expression, VariableExpression ve) {
+        Variable v = ve.getAccessedVariable();
+        boolean isFinal = isFinal(v.getModifiers());
+        boolean isParameter = v instanceof Parameter;
+        if (isFinal && isParameter) {
+            addError("Cannot assign a value to final variable '" + v.getName() + "'", expression);
+        }
+    }
+
     public void visitVariableExpression(VariableExpression expression) {
         String name = expression.getName();
         Variable v = checkVariableNameForDeclaration(name, expression);
@@ -346,20 +392,14 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     public void visitPropertyExpression(PropertyExpression expression) {
-        boolean ipe = inPropertyExpression;
-        inPropertyExpression = true;
         expression.getObjectExpression().visit(this);
-        inPropertyExpression = false;
         expression.getProperty().visit(this);
         checkPropertyOnExplicitThis(expression);
-        inPropertyExpression = ipe;
     }
 
     public void visitClosureExpression(ClosureExpression expression) {
         pushState();
 
-        inClosure = true;
-
         expression.setVariableScope(currentScope);
 
         if (expression.isParameterSpecified()) {
@@ -378,9 +418,18 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         }
 
         super.visitClosureExpression(expression);
+        markClosureSharedVariables();
+
         popState();
     }
 
+    private void markClosureSharedVariables() {
+        VariableScope scope = currentScope;
+        for (Iterator<Variable> it = scope.getReferencedLocalVariablesIterator(); it.hasNext(); ) {
+            it.next().setClosureSharedVariable(true);
+        }
+    }
+
     public void visitCatchStatement(CatchStatement statement) {
         pushState();
         Parameter p = statement.getVariable();
@@ -402,26 +451,34 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     // ------------------------------
 
     public void visitClass(ClassNode node) {
-        // AIC are already done, doing them here again will lead
-        // to wrong scopes
+        // AIC are already done, doing them here again will lead to wrong scopes
         if (node instanceof InnerClassNode) {
             InnerClassNode in = (InnerClassNode) node;
-            if (in.isAnonymous()) return;
+            if (in.isAnonymous() && !in.isEnum()) return;
         }
-        
+
         pushState();
 
-        currentClass = node;
-        currentScope.setClassScope(node);
+        prepareVisit(node);
 
         super.visitClass(node);
         popState();
     }
 
+    /**
+     * Setup the current class node context.
+     * @param node
+     */
+    public void prepareVisit(ClassNode node) {
+        currentClass = node;
+        currentScope.setClassScope(node);
+    }
+
     protected void visitConstructorOrMethod(MethodNode node, boolean isConstructor) {
         pushState(node.isStatic());
         inConstructor = isConstructor;
         node.setVariableScope(currentScope);
+        visitAnnotations(node);
         
         // GROOVY-2156
         Parameter[] parameters = node.getParameters();
@@ -430,8 +487,8 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         }
 
         declare(node.getParameters(), node);
+        visitClassCodeContainer(node.getCode());
 
-        super.visitConstructorOrMethod(node, isConstructor);
         popState();
     }
 
@@ -456,6 +513,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
                 call.setObjectExpression(object);
                 ConstantExpression method = new ConstantExpression("call");
                 method.setSourcePosition(methodNameConstant); // important for GROOVY-4344
+                call.setImplicitThis(false);
                 call.setMethod(method);
             }
 
@@ -479,8 +537,6 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
             visitClosureExpression(cl);
         }
 
-        boolean ic = inClosure;
-        inClosure = true;
         for (FieldNode field : innerClass.getFields()) {
             final Expression expression = field.getInitialExpression();
             if (expression != null) {
@@ -491,10 +547,10 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         for (Statement statement : innerClass.getObjectInitializerStatements()) {
             statement.visit(this);
         }
-        inClosure = ic;
+        markClosureSharedVariables();
         popState();
     }
-    
+
     public void visitProperty(PropertyNode node) {
         pushState(node.isStatic());
         super.visitProperty(node);
@@ -510,10 +566,9 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     public void visitAnnotations(AnnotatedNode node) {
         List<AnnotationNode> annotations = node.getAnnotations();
         if (annotations.isEmpty()) return;
-        ClassNode annType;
         for (AnnotationNode an : annotations) {
-        	// skip built-in properties
-        	if (an.isBuiltIn()) continue;
+            // skip built-in properties
+            if (an.isBuiltIn()) continue;
             for (Map.Entry<String, Expression> member : an.getMembers().entrySet()) {
                 Expression annMemberValue = member.getValue();
                 annMemberValue.visit(this);
