@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2010 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,27 +15,20 @@
  */
 package org.codehaus.groovy.control;
 
-import groovy.lang.GroovyClassLoader;
 import org.codehaus.groovy.ast.*;
 import org.codehaus.groovy.ast.expr.*;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.CatchStatement;
 import org.codehaus.groovy.ast.stmt.ForStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
-import org.codehaus.groovy.classgen.Verifier;
-import org.codehaus.groovy.control.messages.ExceptionMessage;
+import org.codehaus.groovy.control.ClassNodeResolver.LookupResult;
 import org.codehaus.groovy.syntax.Types;
 import org.codehaus.groovy.GroovyBugError;
 import org.objectweb.asm.Opcodes;
 
-import java.io.File;
-import java.io.IOException;
 import java.lang.annotation.Retention;
 import java.lang.annotation.RetentionPolicy;
 import java.lang.reflect.Modifier;
-import java.net.MalformedURLException;
-import java.net.URL;
-import java.net.URLConnection;
 import java.util.*;
 
 /**
@@ -44,7 +37,7 @@ import java.util.*;
  * find the Class for a ClassExpression and prints an error if
  * it fails to do so. Constructions like C[], foo as C, (C) foo
  * will force creation of a ClassExpression for C
- * <p/>
+ * <p>
  * Note: the method to start the resolving is  startResolving(ClassNode, SourceUnit).
  *
  * @author Jochen Theodorou
@@ -56,8 +49,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     // note: BigInteger and BigDecimal are also imported by default
     public static final String[] DEFAULT_IMPORTS = {"java.lang.", "java.io.", "java.net.", "java.util.", "groovy.lang.", "groovy.util."};
     private CompilationUnit compilationUnit;
-    private Map cachedClasses = new HashMap();
-    private static final Object NO_CLASS = new Object();
     private SourceUnit source;
     private VariableScope currentScope;
 
@@ -70,6 +61,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean checkingVariableTypeInDeclaration = false;
     private ImportNode currImportNode = null;
     private MethodNode currentMethod;
+    private ClassNodeResolver classNodeResolver;
 
     /**
      * we use ConstructedClassWithPackage to limit the resolving the compiler
@@ -141,6 +133,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
     public ResolveVisitor(CompilationUnit cu) {
         compilationUnit = cu;
+        this.classNodeResolver = new ClassNodeResolver();
     }
 
     public void startResolving(ClassNode node, SourceUnit source) {
@@ -277,7 +270,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 resolveFromCompileUnit(type) ||
                 resolveFromDefaultImports(type, testDefaultImports) ||
                 resolveFromStaticInnerClasses(type, testStaticInnerClasses) ||
-                resolveToClass(type);
+                resolveToOuter(type);
     }
 
     private boolean resolveNestedClass(ClassNode type) {
@@ -340,69 +333,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         return false;   
     }
 
-    // NOTE: copied from GroovyClassLoader
-    private long getTimeStamp(Class cls) {
-        return Verifier.getTimestamp(cls);
-    }
-
-    // NOTE: copied from GroovyClassLoader
-    private boolean isSourceNewer(URL source, Class cls) {
-        try {
-            long lastMod;
-
-            // Special handling for file:// protocol, as getLastModified() often reports
-            // incorrect results (-1)
-            if (source.getProtocol().equals("file")) {
-                // Coerce the file URL to a File
-                String path = source.getPath().replace('/', File.separatorChar).replace('|', ':');
-                File file = new File(path);
-                lastMod = file.lastModified();
-            } else {
-                URLConnection conn = source.openConnection();
-                lastMod = conn.getLastModified();
-                conn.getInputStream().close();
-            }
-            return lastMod > getTimeStamp(cls);
-        } catch (IOException e) {
-            // if the stream can't be opened, let's keep the old reference
-            return false;
-        }
-    }
-
-
-    private boolean resolveToScript(ClassNode type) {
-        String name = type.getName();
-
-        if (name.startsWith("java.")) return type.isResolved();
-        //TODO: don't ignore inner static classes completely
-        if (name.indexOf('$') != -1) return type.isResolved();
-        ModuleNode module = currentClass.getModule();
-        if (module.hasPackageName() && name.indexOf('.') == -1) return type.isResolved();
-        // try to find a script from classpath
-        GroovyClassLoader gcl = compilationUnit.getClassLoader();
-        URL url = null;
-        try {
-            url = gcl.getResourceLoader().loadGroovySource(name);
-        } catch (MalformedURLException e) {
-            // fall through and let the URL be null
-        }
-        if (url != null) {
-            if (type.isResolved()) {
-                Class cls = type.getTypeClass();
-                // if the file is not newer we don't want to recompile
-                if (!isSourceNewer(url, cls)) return true;
-                // since we came to this, we want to recompile
-                cachedClasses.remove(type.getName());
-                type.setRedirect(null);
-            }
-            SourceUnit su = compilationUnit.addSource(url);
-            currentClass.getCompileUnit().addClassNodeToCompile(type, su);
-            return true;
-        }
-        // type may be resolved through the classloader before
-        return type.isResolved();
-    }
-
     private String replaceLastPoint(String name) {
         int lastPoint = name.lastIndexOf('.');
         name = new StringBuffer()
@@ -668,70 +598,32 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         return false;
     }
 
-    private boolean resolveToClass(ClassNode type) {
+    private boolean resolveToOuter(ClassNode type) {
         String name = type.getName();
 
-        // We use here the class cache cachedClasses to prevent
-        // calls to ClassLoader#loadClass. disabling this cache will
-        // cause a major performance hit. Unlike at the end of this
-        // method we do not return true or false depending on if we
-        // want to recompile or not. If the class was cached, then
-        // we do not want to recompile, recompilation is already
-        // scheduled then
-
-        Object cached = cachedClasses.get(name);
-        if (cached == NO_CLASS)
-            return false;
-
-        if (cached != null) {
-            // cached == SCRIPT should not happen here!
-            type.setRedirect((ClassNode) cached);
-            return true;
-        }
-
-
         // We do not need to check instances of LowerCaseClass
         // to be a Class, because unless there was an import for
-        // for this we  do not lookup these cases. This was a decision
+        // for this we do not lookup these cases. This was a decision
         // made on the mailing list. To ensure we will not visit this
         // method again we set a NO_CLASS for this name
         if (type instanceof LowerCaseClass) {
-            cachedClasses.put(name,NO_CLASS);
+            classNodeResolver.cacheClass(name, ClassNodeResolver.NO_CLASS);
             return false;
         }
 
         if (currentClass.getModule().hasPackageName() && name.indexOf('.') == -1) return false;
-        GroovyClassLoader loader = compilationUnit.getClassLoader();
-        Class cls;
-        try {
-            // NOTE: it's important to do no lookup against script files
-            // here since the GroovyClassLoader would create a new CompilationUnit
-            cls = loader.loadClass(name, false, true);
-        } catch (ClassNotFoundException cnfe) {
-            cachedClasses.put(name, NO_CLASS);
-            return resolveToScript(type);
-        } catch (CompilationFailedException cfe) {
-            compilationUnit.getErrorCollector().addErrorAndContinue(new ExceptionMessage(cfe, true, source));
-            return resolveToScript(type);
+        LookupResult lr = null;
+        lr = classNodeResolver.resolveName(name, compilationUnit);
+        if (lr!=null) {
+            if (lr.isSourceUnit()) {
+                SourceUnit su = lr.getSourceUnit();
+                currentClass.getCompileUnit().addClassNodeToCompile(type, su);
+            } else {
+                type.setRedirect(lr.getClassNode());
+            }
+            return true;
         }
-        //TODO: the case of a NoClassDefFoundError needs a bit more research
-        // a simple recompilation is not possible it seems. The current class
-        // we are searching for is there, so we should mark that somehow.
-        // Basically the missing class needs to be completely compiled before
-        // we can again search for the current name.
-        /*catch (NoClassDefFoundError ncdfe) {
-            cachedClasses.put(name,SCRIPT);
-            return false;
-        }*/
-        if (cls == null) return false;
-        ClassNode cn = ClassHelper.make(cls);
-        cachedClasses.put(name, cn);
-        type.setRedirect(cn);
-        //NOTE: we might return false here even if we found a class,
-        //      because  we want to give a possible script a chance to
-        //      recompile. This can only be done if the loader was not
-        //      the instance defining the class.
-        return cls.getClassLoader() == loader || resolveToScript(type);
+        return false;
     }
 
 
@@ -980,6 +872,13 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         return !Character.isLowerCase(name.charAt(0));
     }
 
+    private boolean isLeftSquareBracket(int op) {
+        return op == Types.ARRAY_EXPRESSION
+                || op == Types.LEFT_SQUARE_BRACKET
+                || op == Types.SYNTH_LIST
+                || op == Types.SYNTH_MAP;
+    }
+
     protected Expression transformBinaryExpression(BinaryExpression be) {
         Expression left = transform(be.getLeftExpression());
         int type = be.getOperation().getType();
@@ -993,7 +892,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             addError(error, be.getLeftExpression());
             return be;
         }
-        if (left instanceof ClassExpression) {
+        if (left instanceof ClassExpression && isLeftSquareBracket(type)) {
             if (be.getRightExpression() instanceof ListExpression) {
                 ListExpression list = (ListExpression) be.getRightExpression();
                 if (list.getExpressions().isEmpty()) {
@@ -1411,4 +1310,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         resolveGenericsTypes(type.getGenericsTypes());
         genericsType.setResolved(genericsType.getType().isResolved());
     }
+
+    public void setClassNodeResolver(ClassNodeResolver classNodeResolver) {
+        this.classNodeResolver = classNodeResolver;
+    }
 }
