@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2008 the original author or authors.
+ * Copyright 2003-2012 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,17 @@
  */
 package org.codehaus.groovy.runtime.typehandling;
 
+import groovy.lang.Closure;
 import groovy.lang.GString;
 import groovy.lang.GroovyRuntimeException;
 import org.codehaus.groovy.reflection.ReflectionCache;
+import org.codehaus.groovy.reflection.stdclasses.CachedSAMClass;
 import org.codehaus.groovy.runtime.*;
 
 import java.io.File;
 import java.io.IOException;
 import java.lang.reflect.Array;
+import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.math.BigDecimal;
 import java.math.BigInteger;
@@ -43,7 +46,7 @@ public class DefaultTypeTransformation {
     //  --------------------------------------------------------       
     
     public static byte byteUnbox(Object value) {
-        Number n = castToNumber(value);
+        Number n = castToNumber(value, byte.class);
         return n.byteValue();
     }
 
@@ -52,12 +55,12 @@ public class DefaultTypeTransformation {
     }
 
     public static short shortUnbox(Object value) {
-        Number n = castToNumber(value);
+        Number n = castToNumber(value, short.class);
         return n.shortValue();
     }
 
     public static int intUnbox(Object value) {
-        Number n = castToNumber(value);
+        Number n = castToNumber(value, int.class);
         return n.intValue();
     }
 
@@ -66,17 +69,17 @@ public class DefaultTypeTransformation {
     }
 
     public static long longUnbox(Object value) {
-        Number n = castToNumber(value);
+        Number n = castToNumber(value, long.class);
         return n.longValue();
     }
 
     public static float floatUnbox(Object value) {
-        Number n = castToNumber(value);
+        Number n = castToNumber(value, float.class);
         return n.floatValue();
     }
 
     public static double doubleUnbox(Object value) {
-        Number n = castToNumber(value);
+        Number n = castToNumber(value, double.class);
         return n.doubleValue();
     } 
 
@@ -117,8 +120,8 @@ public class DefaultTypeTransformation {
     }
     
     public static Number castToNumber(Object object) {
-    	// default to Number class in exception details, else use the specified Number subtype.
-    	return castToNumber(object, Number.class);
+        // default to Number class in exception details, else use the specified Number subtype.
+        return castToNumber(object, Number.class);
     }
     
     public static Number castToNumber(Object object, Class type) {
@@ -127,12 +130,19 @@ public class DefaultTypeTransformation {
         if (object instanceof Character) {
             return Integer.valueOf(((Character) object).charValue());
         }
+        if (object instanceof GString) {
+            String c = ((GString) object).toString();
+            if (c.length() == 1) {
+                return Integer.valueOf(c.charAt(0));
+            } else {
+                throw new GroovyCastException(c, type);
+            }
+        }
         if (object instanceof String) {
             String c = (String) object;
             if (c.length() == 1) {
                 return Integer.valueOf(c.charAt(0));
-            }
-            else {
+            } else {
                 throw new GroovyCastException(c, type);
             }
         }
@@ -151,8 +161,13 @@ public class DefaultTypeTransformation {
         if (object == null) {
             return false;
         }
-
-        // if the object is not null, try to call an asBoolean() method on the object
+        
+        // equality check is enough and faster than instanceof check, no need to check superclasses since Boolean is final
+        if (object.getClass() == Boolean.class) {   
+            return ((Boolean)object).booleanValue();
+        }
+        
+        // if the object is not null and no Boolean, try to call an asBoolean() method on the object
         return (Boolean)InvokerHelper.invokeMethod(object, "asBoolean", InvokerHelper.EMPTY_ARGS);
     }
     
@@ -202,6 +217,8 @@ public class DefaultTypeTransformation {
             if (aClass.isArray()) {
                 if (type.isAssignableFrom(ArrayList.class) && (Modifier.isAbstract(modifiers) || Modifier.isInterface(modifiers))) {
                     answer = new ArrayList();
+                } else if (type.isAssignableFrom(HashSet.class) && (Modifier.isAbstract(modifiers) || Modifier.isInterface(modifiers))) {
+                    answer = new HashSet();
                 } else {
                     // let's call the collections constructor
                     // passing in the list wrapper
@@ -293,6 +310,10 @@ public class DefaultTypeTransformation {
                 }
                 return answer;
             }
+        } else if (object instanceof String && type.isEnum()) {
+            return Enum.valueOf(type, (String) object);
+        } else if (object instanceof GString && type.isEnum()) {
+            return Enum.valueOf(type, object.toString());
         }
         Object[] args = null;
         if (object instanceof Collection) {
@@ -307,25 +328,56 @@ public class DefaultTypeTransformation {
             args = new Object[1];
             args[0] = object;
         }
+        
+        if (object instanceof Closure) {
+            Method m = CachedSAMClass.getSAMMethod(type);
+            if (m!=null) {
+                return CachedSAMClass.coerceToSAM((Closure) object, m, type, type.isInterface());
+            }
+        }
+        
+        Exception nested = null;
         if (args != null) {
             try {
                 return InvokerHelper.invokeConstructorOf(type, args);
             } catch (InvokerInvocationException iie){
                 throw iie;
+            } catch (GroovyRuntimeException e) {
+                if(e.getMessage().contains("Could not find matching constructor for")) {
+                    try {
+                        return InvokerHelper.invokeConstructorOf(type, object);
+                    } catch (InvokerInvocationException iie){
+                        throw iie;
+                    } catch (Exception ex) {
+                        // let's ignore exception and return the original object
+                        // as the caller has more context to be able to throw a more
+                        // meaningful exception (but stash to get message later)
+                        nested = e;
+                    }
+                } else {
+                    nested = e;
+                }
             } catch (Exception e) {
                 // let's ignore exception and return the original object
                 // as the caller has more context to be able to throw a more
-                // meaningful exception
+                // meaningful exception (but stash to get message later)
+                nested = e;
             }
         }
-        throw new GroovyCastException(object,type);
+        GroovyCastException gce;
+        if (nested != null) {
+            gce = new GroovyCastException(object, type, nested);
+        } else {
+            gce = new GroovyCastException(object, type);
+        }
+        throw gce;
     }
 
     private static Class castToClass(Object object) {
         try {
-            return Class.forName (object.toString());
+            return Class.forName(object.toString());
         } catch (Exception e) {
-            throw new GroovyCastException(object,Class.class);
+            throw new GroovyCastException(object, Class.class, e);
         }
     }
 
@@ -422,14 +474,14 @@ public class DefaultTypeTransformation {
             return adapter.asList();
         }
         else if (value instanceof String) {
-            return DefaultGroovyMethods.toList((String) value);
+            return StringGroovyMethods.toList((String) value);
         }
         else if (value instanceof GString) {
-            return DefaultGroovyMethods.toList(value.toString());
+            return StringGroovyMethods.toList(value.toString());
         }
         else if (value instanceof File) {
             try {
-                return DefaultGroovyMethods.readLines((File) value);
+                return ResourceGroovyMethods.readLines((File) value);
             }
             catch (IOException e) {
                 throw new GroovyRuntimeException("Error reading file: " + value, e);
@@ -593,6 +645,17 @@ public class DefaultTypeTransformation {
         if (left instanceof List && right instanceof List) {
             return DefaultGroovyMethods.equals((List) left, (List) right);
         }
+        if (left instanceof Map.Entry && right instanceof Map.Entry) {
+            Object k1 = ((Map.Entry)left).getKey();
+            Object k2 = ((Map.Entry)right).getKey();
+            if (k1 == k2 || (k1 != null && k1.equals(k2))) {
+                Object v1 = ((Map.Entry)left).getValue();
+                Object v2 = ((Map.Entry)right).getValue();
+                if (v1 == v2 || (v1 != null && DefaultTypeTransformation.compareEqual(v1, v2)))
+                    return true;
+            }
+            return false;
+        }
         return ((Boolean) InvokerHelper.invokeMethod(left, "equals", right)).booleanValue();
     }
 
@@ -805,14 +868,14 @@ public class DefaultTypeTransformation {
     }
     
     public static Character getCharFromSizeOneString(Object value) {
-    	if (value instanceof GString) value = value.toString();
+        if (value instanceof GString) value = value.toString();
         if (value instanceof String) {
             String s = (String) value;
             if (s.length() != 1) throw new IllegalArgumentException("String of length 1 expected but got a bigger one");
             return new Character(s.charAt(0));
         } else {
             return ((Character) value);
-        }    	
+        }
     }
 
 }
