@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2013 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -63,7 +63,11 @@ public final class LongMath extends NumberMath {
     protected Number unaryMinusImpl(Number left) {
         return Long.valueOf(-left.longValue());
     }
-    
+
+    protected Number unaryPlusImpl(Number left) {
+        return Long.valueOf(left.longValue());
+    }
+
     protected Number bitwiseNegateImpl(Number left) {
         return Long.valueOf(~left.longValue());
     }
