@@ -1,4 +1,4 @@
-package somepackage;
+package stubgenerator.propertyUsageFromJava.somepackage;
 
 public class JavaBean {
     public void usePogo() {
@@ -8,4 +8,4 @@ public class JavaBean {
         System.out.println(pogo.getName());
         System.out.println(pogo.getAge());
     }
-}
\ No newline at end of file
+}
