@@ -27,7 +27,7 @@ import org.objectweb.asm.*;
 import java.util.HashMap;
 import java.util.Map;
 
-public class GenericsTestBase extends GroovyTestCase {
+public abstract class GenericsTestBase extends GroovyTestCase {
     MyLoader loader;
     HashMap signatures = new HashMap();
     
@@ -52,9 +52,9 @@ public class GenericsTestBase extends GroovyTestCase {
             return super.createClass(code,classNode);
         }        
     }
-    private class GenericsTester  extends ClassAdapter {
+    private class GenericsTester extends ClassVisitor {
         public GenericsTester(ClassVisitor cv) {
-            super(cv);
+            super(Opcodes.ASM4,cv);
         }
         public void visit(int version, int access, String name,
                 String signature, String superName, String[] interfaces) {
