@@ -24,9 +24,9 @@ import java.util.Iterator;
  * Base class for test cases
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 21516 $
+ * @version $Revision$
  */
-public class TestSupport extends GroovyTestCase {
+public abstract class TestSupport extends GroovyTestCase {
 
     public String[] getMockArguments() {
         return new String[]{"a", "b", "c"};
