@@ -1,3 +1,18 @@
+/*
+ * Copyright 2003-2012 the original author or authors.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package groovy.inspect;
 
 import groovy.lang.GroovyShell;
@@ -88,9 +103,9 @@ public class InspectorTest extends MockObjectTestCase implements Serializable {
         String[] names = {"sleep", "sleep", "println", "println", "println", "find", "find", "findResult", "findResult",
                 "print", "print", "each", "invokeMethod", "asType", "inspect", "is", "isCase", "identity", "getAt",
                 "putAt", "dump", "getMetaPropertyValues", "getProperties", "use", "use", "use", "printf", "printf",
-                "eachWithIndex", "every", "every", "any", "any", "grep", "grep", "collect", "collect", "findAll","findAll",
+                "eachWithIndex", "every", "every", "any", "any", "grep", "grep", "collect", "collect", "collect", "findAll","findAll",
                 "split", "findIndexOf", "findIndexOf", "findLastIndexOf", "findLastIndexOf", "findIndexValues", "findIndexValues",
-                "iterator", "addShutdownHook", "sprintf", "sprintf", "with", "inject", "getMetaClass", "setMetaClass",
+                "iterator", "addShutdownHook", "sprintf", "sprintf", "with", "inject", "inject", "getMetaClass", "setMetaClass",
                 "metaClass", "respondsTo", "respondsTo", "hasProperty", "toString", "asBoolean"
         };
         assertEquals("Incorrect number of methods found examining: " + getNamesFor(metaMethods), names.length, metaMethods.length);
@@ -178,7 +193,7 @@ public class InspectorTest extends MockObjectTestCase implements Serializable {
         assertEquals(5, fields.length); // 3 from JMock
         String[] names = {"someField", "SOME_CONST", "ANYTHING", "NULL", "NOT_NULL"};
         assertNameEquals(names, fields);
-        String[] details = {"JAVA", "public", "InspectorTest", "String", "someField", "\"only for testing\""};
+        String[] details = {"JAVA", "public", "InspectorTest", "String", "someField", "'only for testing'"};
         assertContains(fields, details);
     }
 
@@ -248,7 +263,7 @@ public class InspectorTest extends MockObjectTestCase implements Serializable {
     }
 
     private String concat(String[] details) {
-        StringBuffer detailBuffer = new StringBuffer();
+        StringBuilder detailBuffer = new StringBuilder();
         for (int i = 0; i < details.length; i++) {
             detailBuffer.append(details[i]);
             detailBuffer.append(" ");
