@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2011 the original author or authors.
+ * Copyright 2003-2012 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,8 @@
 package org.codehaus.groovy.runtime;
 
 import groovy.lang.MissingMethodException;
-import groovy.util.GroovySwingTestCase;
+import groovy.util.GroovyTestCase;
+import groovy.util.HeadlessTestSupport;
 import groovy.util.Node;
 
 import javax.swing.*;
@@ -30,9 +31,9 @@ import java.util.Map;
  * Test the property access of the Invoker class
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 21516 $
+ * @version $Revision$
  */
-public class PropertyTest extends GroovySwingTestCase {
+public class PropertyTest extends GroovyTestCase {
 
     public void testMapProperties() throws Exception {
         Map map = new HashMap();
@@ -54,22 +55,6 @@ public class PropertyTest extends GroovySwingTestCase {
         assertGetSetProperty(bean, "dynamicFoo", "aValue", "NewValue");
     }
 
-    /**
-     * todo this is no longer possible in new groovy
-     * public void testUsingMethodProperty() throws Exception {
-     * DummyBean bean = new DummyBean();
-     * <p/>
-     * assertGetSetProperty(bean, "name", "James", "Bob");
-     * <p/>
-     * Object value = InvokerHelper.getProperty(bean, "getName");
-     * assertTrue("Should have returned a closure: " + value, value instanceof Closure);
-     * Closure closure = (Closure) value;
-     * Object result = closure.call(null);
-     * assertEquals("Result of call to closure", "Bob", result);
-     * }
-     */
-
-
     public void testStaticProperty() throws Exception {
         Object value = InvokerHelper.getProperty(System.class, "out");
         assertEquals("static property out", System.out, value);
@@ -94,19 +79,6 @@ public class PropertyTest extends GroovySwingTestCase {
         assertEquals("value property", "x", value);
     }
 
-    /**
-     * todo this is no longer possible in new groovy
-     * public void testMethodProperty() throws Exception {
-     * Object value = InvokerHelper.getProperty(this, "getCheese");
-     * assertTrue("Should have returned a closure: " + value, value instanceof Closure);
-     * <p/>
-     * Object result = ((Closure) value).call();
-     * assertEquals("result of closure call", getCheese(), result);
-     * <p/>
-     * System.out.println("Closure: " + value + " and cheese: " + result);
-     * }
-     */
-
     public void testListCoercionProperty() throws Exception {
         DummyBean bean = new DummyBean();
         List list = new ArrayList();
@@ -118,7 +90,7 @@ public class PropertyTest extends GroovySwingTestCase {
     }
 
     public void testListCoercionPropertyOnJFrame() throws Exception {
-        if (isHeadless()) return;
+        if (HeadlessTestSupport.isHeadless()) return;
 
         try {
             JFrame bean = new JFrame();
@@ -183,12 +155,11 @@ public class PropertyTest extends GroovySwingTestCase {
     }
 
     public void testUsingInPropertyOnProcessViaGroovyMethod() throws Exception {
-        Process process = DefaultGroovyMethods.execute(System.getProperty("java.home") + "/bin/java -version");
+        String[] java = {System.getProperty("java.home") + "/bin/java", "-version"};
+        Process process = ProcessGroovyMethods.execute(java);
         Object value = InvokerHelper.getProperty(process, "in");
         assertNotNull(value);
-
         System.out.println("Found in: " + value);
-
         process.destroy();
     }
 
@@ -197,7 +168,7 @@ public class PropertyTest extends GroovySwingTestCase {
     }
 
     public void testComponentParent() {
-        if (isHeadless()) return;
+        if (HeadlessTestSupport.isHeadless()) return;
 
         JPanel panel = new JPanel();
         JButton bean = new JButton();
